%mousetriangle
% Function which divides up a figure window into four coloured triangles.
% The intersection of the triangles is determined dynamically by moving the
% mouse.
%
% LAST UPDATED by Andy French. November 2012

function mousetriangle

%Create figure and axes
fig = figure('name','Mousetriangle','WindowButtonDownFcn',@clickmouse,...
    'Toolbar','none','menubar','none','numbertitle','off');
ax = axes('units','normalized','outerposition',[0,0,1,1],'position',[0,0,1,1]);

%Get screensize and make figure fill the screen
screensize = get(0,'screensize');
set(fig,'units','pixels','position',screensize);

%Get GUI user data
gui = get(fig,'UserData');

%Set default light z position
gui.light_z_default = 0.4;

%Make triangular patches and store handles to these in GUI user data
gui.p1 = patch([0,0.5,0],[0,0.5,1],'r','edgecolor','none','facelighting','gouraud','SpecularExponent',5);
gui.p2 = patch([0,1,0.5],[0,0,0.5],'g','edgecolor','none','facelighting','gouraud','SpecularExponent',5);
gui.p3 = patch([1,1,0.5],[0,1,0.5],'b','edgecolor','none','facelighting','gouraud','SpecularExponent',5);
gui.p4 = patch([1,0,0.5],[1,1,0.5],'y','edgecolor','none','facelighting','gouraud','SpecularExponent',5);

%Create (invisible) light object. It will be turned on by right mouse clicks
gui.light = light('visible','off','position',[0.5,0.5,gui.light_z_default],'style','local');

%Update GUI user data
set(fig,'UserData',gui);

%%

%Function which runs when the mouse is clicked inside the figure window
function clickmouse(src,evnt)

%Get GUI user data
gui = get(gcf,'UserData');

%Remove mouse pointer
set(gcf,'PointerShapeCData',nan(16,16),'Pointer','custom');

%Set callbacks for GUI
thisfig = gcbf();
set(thisfig,'WindowButtonMotionFcn',@dragmouse,'WindowButtonUpFcn',@unclickmouse);

%Start clock and display time elapsed in title bar
gui.t0 = cputime;
set(gcf,'name','mousetriangle')

%Set flag in gui which describes the type of mouse click
button = get(gcf, 'SelectionType');

%Double left mouse button click
if strcmp(button,'open')
    gui.button = 'double click';
    
    %Single left mouse click
elseif strcmp(button,'normal')
    gui.button = 'single click';
    
    %Shift + click or right & left mouse click
elseif strcmp(button,'extend')
    gui.button = 'shift & click';
    
    %Turn on light, but change colour to green
    set( gui.light,'visible','on','color',[0.1,1,0.2]);
    
    %Modify figure name
    set(gcf,'name','mousetriangle LET THERE BE SPOOKY LIGHT!')
       
    %Control + click or right mouse click
elseif strcmp(button,'alt')
    gui.button = 'ctrl & click';
    
    %Turn on light
    set( gui.light,'visible','on');
    
    %Modify figure name
    set(gcf,'name','mousetriangle LET THERE BE LIGHT!')
else
    gui.button='nothing';
end

%Store position when mouse was clicked
pos = get(gca,'CurrentPoint');
x = pos(1,1); y = pos(1,2);
gui.start_x = x; gui.start_y = y;

%Update GUI user data
set(gcf,'UserData',gui);

%%

%Function which runs when the mouse is dragged in the figure window
function dragmouse(src,evnt)

%Get GUI user data
gui = get(gcf,'UserData');

%Get current position of the mouse
pos = get(gca,'CurrentPoint');
x = pos(1,1); y = pos(1,2);

%Update time in figure title bar
if strcmp(gui.button,'ctrl & click') == 1
    set(gcf,'name',['mousetriangle LET THERE BE LIGHT! dragging time = ',num2str( cputime - gui.t0 ),'s'] );
elseif strcmp(gui.button,'shift & click') == 1
    set(gcf,'name',['mousetriangle LET THERE BE SPOOKY LIGHT! dragging time = ',num2str( cputime - gui.t0 ),'s'] );
else
    set(gcf,'name',['mousetriangle dragging time = ',num2str( cputime - gui.t0 ),'s'] );
end

%Update triangle patches such that the centre is where the mouse is. Only
%do this iof the current mouse position is inside the figure
if (x>=0) && (x<=1) && (y>=0) && (y<=1)
    set(gui.p1,'XData', [0,x,0],'YData', [0,y,1] );
    set(gui.p2,'XData', [0,1,x],'YData', [0,0,y] );
    set(gui.p3,'XData', [1,1,x],'YData', [0,1,y] );
    set(gui.p4,'XData', [1,0,x],'YData', [1,1,y] );
    
    %Modify light object, correct button combination is pressed
    if (strcmp(gui.button,'ctrl & click') == 1) || (strcmp(gui.button,'shift & click') == 1)
        set( gui.light,'position',[x,y,gui.light_z_default]);
    end
    
    %Flush the graphics buffer and update GUI user data
    drawnow;
    set(gcf,'UserData',gui);
end

%%

%Function which executes when the mouse button is released
function unclickmouse(src,evnt)

%Get handle to figure currently invoking a callback
thisfig = gcbf();

%Get GUI data
gui = get(thisfig,'UserData');

%Reset mouse pointer to arrow. Reset mouse dragging functions to do nothing
set(thisfig,'Pointer','arrow','WindowButtonUpFcn','','WindowButtonMotionFcn','');

%Reset light colour to white
set(gui.light,'color',[1 1 1]);

%Reset clock
gui.t0 = 0;

%Modify figure name
set(thisfig,'name','mousetriangle')

%Turn off light and update figure UserData
set( gui.light,'visible','off');
set(thisfig,'UserData',gui);

%End of code