%circle_stitch
% Curve stitches two circles.
%
% LAST UPDATED by Andy French April 2012

function circle_stitch

%Circle radii and centres

%Circle 1
xc1 = 2;
yc1 = 2;
R1 = 1;

%Circle 2
xc2 = 0;
yc2 = 0;
R2 = 5;
phase = pi/3;
flip = 1;

%Number of stitches per circle
S = 72;

%Output image DPI
DPI = 300;

%Define colormap string
cmap_str = 'prism';

%Plot circle outlines?
plot_circle_outline = 1;

%Number of coordinates per circle
M = 300;

%%

%Create output image name
name = ['Two circles with ',num2str(S),' stitches'];

%Define x,y,z coordinates of circles
t = linspace(0,2*pi,M+1).';
X1 = xc1+ R1*sin(t);
Y1 = yc1 + R1*cos(t);
Z1 = zeros(M+1,1);
X2 = xc2+ R2*sin(t-phase);
Y2 = yc2 + R2*cos(t-phase);
Z2 = zeros(M+1,1);

%Stitch circles
if flip==1
    [x,y,z] = curve_stitch( S, X1,Y1,Z1, flipud(X2),flipud(Y2),flipud(Z2) );
else
    [x,y,z] = curve_stitch( S, X1,Y1,Z1, X2,Y2,Z2 );
end

%Create figure
figure('color',[1 1 1],'name','Circle stitch');

%Plot curves and stitches
if plot_circle_outline==1
    plot(X1,Y1,'k','linewidth',2);
    hold on
    plot(X2,Y2,'k','linewidth',2);
end
plot3(x,y,z,'r','linewidth',0.01);
axis equal
view(2)
xlabel('x')
ylabel('y')
zlabel('z')
title(name)
axis off

%Print PNG output
print(gcf,'-dpng',['-r',num2str(DPI)], name )

%%

%curve_stitch
% Function which 'stitches' two curves with straight line segments.
% Curves are defined by column vectors of X,Y,Z coordinates.
%
% LAST UPDATED by Andy French April 2016

function [x,y,z] = curve_stitch( N, X1,Y1,Z1, X2,Y2,Z2 )

%Mark curves into N equal length pieces
[x1,y1,z1] = curvemark( N, X1,Y1,Z1 );
[x2,y2,z2] = curvemark( N, X2,Y2,Z2 );

%Join marks by line segments
x = [];
y = [];
z = [];
for n=1:N
    x = [ x,[x1(n);x2(n)] ];
    y = [ y,[y1(n);y2(n)] ];
    z = [ z,[z1(n);z2(n)] ];
end

%%

%curvemark
% Function which determines the x,y,z coordinates of N equally spaced
% points along the length of a curve defined in linear line segments
% between coordniates stored in (equal dimension) vectors X,Y,Z.
%
% LAST UPDATED by Andy French April 2012
%
% Syntax: [x,y,z] = curvemark( N, X,Y,Z )
%
% N is the number of points. Must be an integer!
% X,Y,Z vectors define the curve to be marked. They are all column
% vectors of dimensions [ M,1 ]. Note plot3(X,Y,Z) would plot the curve.
%
% x,y,z are corresponding column vectors of coordinates corresponding to N equally
% spaced points along the length of the curve defined by X,Y,Z.

function [x,y,z] = curvemark( N, X,Y,Z )

%Step through curve X,Y,Z and cumulatively work out total line length
M = length(X);
L = zeros(M,1);
for m=2:M
    L(m) = L(m-1) + sqrt( (X(m)-X(m-1))^2 + (Y(m)-Y(m-1))^2 + (Z(m)-Z(m-1))^2 );
end

%Interpolate L in N equal steps
Li = interp1( (1:M).', L, linspace( 1, M , N).' );

%Initialize outputs
x = zeros(N,1);
y = zeros(N,1);
z = zeros(N,1);

%Step through Li and (i) find indices m, m+1 of L s.t. L(m) <= Li(n) <=
% L(m+1).
[m,Lm] = approxfind( L,Li );

%Determine coordinates x,y,z from curve X,Y,Z via interpolation
for n=1:N
    dL = Li(n) - Lm(n);
    if dL == 0
        x(n) = X( m(n) );
        y(n) = Y( m(n) );
        z(n) = Z( m(n) );
    elseif dL > 0
        f = ( Li(n) - Lm(n) ) / ( L( m(n) + 1 ) - L( m(n) ) )  ; 
        dX = X( m(n)+1 ) - X( m(n) );
        dY = Y( m(n)+1 ) - Y( m(n) );
        dZ = Z( m(n)+1 ) - Z( m(n) );
        x(n) = X( m(n) ) + f*dX ;
        y(n) = Y( m(n) ) + f*dY ;
        z(n) = Z( m(n) ) + f*dZ ;
    else
        f = ( Li(n) - Lm(n) ) / ( L( m(n) ) - L( m(n) - 1 ) )   ; 
        dX = X( m(n) ) - X( m(n) - 1 );
        dY = Y( m(n) ) - Y( m(n) - 1 );
        dZ = Z( m(n) ) - Z( m(n) - 1 );
        x(n) = X( m(n) ) + f*dX ;
        y(n) = Y( m(n) ) + f*dY ;
        z(n) = Z( m(n) ) + f*dZ ;
    end
end

%%

%approxfind
% Function which returns the index of a vector x whose value is closest to a
% given vector of values v.
%
% LAST UPDATED by Andy French April 2012.

function [ix,xv] = approxfind(x,v)
ix = ones(size(v));
xv = zeros(size(v));
for n=1:length(v)
    z=abs(x-v(n));
    iix=find(min(z)==z);
    if isempty(iix)
        ix(n) = NaN;
        xv(n) = NaN;
    else
        ix(n) = iix(1);
        xv(n) = x( ix(n) );
    end    
end

%%

function approxfind_test

N = 100;
x = linspace(0,2*pi,N);
y = sin(x);
xx = x + 2*rand(1,N);
[ix,xv] = approxfind(x,xx);
figure('color',[1 1 1],'name','approxfind test')
plot(x,y,'b')
hold on;
plot(xv,sin(xv),'r*');
xlabel('x')
ylabel('y')
title('approxfind test')

%%

function curvemark_test

N = 40;
M = 200;

%Define curve parameterically
t = linspace(0,2*pi,M) + rand(1,M);
t = sort(t);
X = cos(t);
Y = sin(t);
Z = t/3;

%Turn coordinates into column vectors to enable straightforward plotting
X = X.';
Y = Y.';
Z = Z.';

%Mark curve into N equal length pieces
[x,y,z] = curvemark( N, X,Y,Z );

%Plot marked curve
figure('color',[1 1 1],'name','curvemark test')
plot3(X,Y,Z,'b');
hold on
plot3(x,y,z,'r*');
axis vis3d
axis equal
grid on
xlabel('x')
ylabel('y')
zlabel('z')
title('Curvemark test')

%End of code