%circlestepsequence
% A circle with N equally separated points is constructed. Straight line
% are drawn between the points in a repeated step sequence.
%
% LAST UPDATED by Andy French April 2016

function circlestepsequence

%Define number of equally spaced points on the circle
N = 14;

%Define step sequence
s = [2,-3,6];

%

%Create x,y vectors of step sequence lines
[x,y] = circle_step_sequence( N, s );

%Plot step sequence upon the unit circle
tt = linspace(0, 2*pi, 300);
xx = sin(tt);
yy = cos(tt);
figure('color',[1 1 1],'name','circle step sequence','numbertitle','off')
plot(xx,yy,'k'); hold on;
plot( x,y,'b')
axis equal
axis off

%Add numbers
t = linspace( 2*pi/N, 2*pi, N );
xt = 1.06*sin(t);
yt = 1.06*cos(t);
for nn=1:N
    text( xt(nn),yt(nn),num2str(nn),...
        'Horizontalalignment','center','rotation',-t(nn)*180/pi,'fontsize',8 );
end

%Plot PNG file
print( gcf, ['stepsequence ',...
    strrep(datestr(now),':','-'),'.png'],'-dpng','-r300' );

%%

%circle_step_sequence
% Function which returns line elements to enable a circular step sequence
% to be plotted.
%
% Syntax: [x,y] = circle_step_sequence( N, s, num_steps )
%
% N is the number of points around a unit circle which form the basis of
% the epicycloid.
% s is a vector of steps representing the step sequence. e.g. s = [1,-3]
% will cause a line to be drawn from the starting point to the next one,
% clockwise, then a line from this point -3 points anticlockwise. The
% sequence will then repeat.
%
% x and y are 2 x K  matrices which describe the line segments which
% describe the rose. The first row is the starting coordinate and the
% second row is the end coordinate of each line segment.

function [x,y] = circle_step_sequence( N, s )

%% Set maximum number of sequences %%
max_sequences = 1000;

%

%Initialise outputs
x = [];
y = [];

%Create x,y points which define the circle
t = linspace( 2*pi/N, 2*pi, N );
xx = sin(t);
yy = cos(t);

%Compute line element pairs
s1 = 1;
back_to_start = 0;
sn = 0;
while (back_to_start == 0) && ( sn < max_sequences )
    for m = 1:length(s)
        %Determine index of where line element is drawn to
        s2 = s1 + s(m);
        s2 = mod( s2-1, N ) + 1;
        
        %Update outputs
        x = [x, [xx(s1); xx(s2)] ];
        y = [y, [yy(s1); yy(s2)] ];
        
        %Update starting position
        s1 = s2;
    end
    
    %Check whether sequence has returned to s1 = 1. If so, terminate the
    %loop.
    if s1==1
        back_to_start=1;
    end
    
    %Update sequence number
    sn = sn + 1;
end

%End of code

