%mondrian
%
% Function which randomly generates an abstract art form in the style
% of Piet Mondrian (1872-1944)
% http://en.wikipedia.org/wiki/Piet_Mondrian.
%
% Uses Line Intersection computation algorthim implemented by U. Murat Erdem.
%
% LAST UPDATED by Andy French. August 2014

function mondrian

%% Inputs %%

%Define maximum number of horizontal and vertical lines
N = 12;

%Define number of rectangles to colour
M = 2*N;

%Define rectangle colours
red = [ 1, 0, 0];
yellow = [ 1, 0.8, 0];
blue = [ 0, 0, 1];

%Define thickness of horizontal and vertical lines
t = 4;

%Define aspect ratio (height / width) of image
phi = ( 1 + sqrt(5) )/2;
a = 1/phi;
a = 1;

%Note rectangular bounding box will have x dimensions [0,1] and y
%dimensions [0, aspect]

%Set minimum line spacing. Try and set d < min( [1,a] )/ N, otherwise
%the program will take a very long time to run!
d = 0.8 * min( [1,a] )/ N;

%%

%Determine x values of vertical lines and y values of horizontal lines,
%each with minimum spacing d
x = random_cut( 0, 1, N , d );
y = random_cut( 0, a, N , d );

%Split these into two parts for (i) full length cut lines and (ii)
%connecting lines
[x1,x2] = random_split( x );
[y1,y2] = random_split( y );

% Horizontal cut lines
for n= 1 : length(y1)
    H(n).xstart = 0;
    H(n).xend = 1;
    H(n).ystart = y1(n);
    H(n).yend = H(n).ystart;
end

% Vertical cut lines
for n= 1 : length(x1)
    V(n).xstart = x1(n);
    V(n).xend = V(n).xstart;
    V(n).ystart = 0;
    V(n).yend = a;
end

% Horizontal connecting lines
if ~isempty(y2)
    for n= 1 : length(y2)
        
        %Set y coordinates of vertical lines
        Hc(n).ystart = y2(n);
        Hc(n).yend = Hc(n).ystart;
        
        if numel(V)>1
            %Choose one of the vertical lines
            r = randi( length(V) , 1 );
            
            %Choose another line number which is not r
            others = 1:length(V);
            others( others == r ) = [];
            s = others( randi( length(V)-1 , 1 ) );
            
            %Set x coordinates of connecting line
            Hc(n).xstart = V(r).xstart;
            Hc(n).xend = V(s).xend;
        else
            %Less than 2 vertical lines, so let a connecting line count as a
            %horizontal cut
            Hc(n).xstart = 0;
            Hc(n).xend = 1;
        end
    end
else
    Hc(1).xstart = [];
    Hc(1).xend = [];
    Hc(1).ystart = [];
    Hc(1).yend = [];
end

% Vertical connecting lines
if ~isempty(x2)
    for n= 1 : length(x2)
        
        %Set x coordinates of vertical lines
        Vc(n).xstart = x2(n);
        Vc(n).xend = Vc(n).xstart;
        
        if numel(H)>1
            %Choose one of the horizontal lines
            r = randi( length(H) , 1 );
            
            %Choose another line number which is not r
            others = 1:length(H);
            others( others == r ) = [];
            s = others( randi( length(H)-1 , 1 ) );
            
            %Set y coordinates of connecting line
            Vc(n).ystart = H(r).ystart;
            Vc(n).yend = H(s).yend;
        else
            %Less than 2 horizontal lines, so let a connecting line count as a
            %vertical cut
            Vc(n).ystart = 0;
            Vc(n).yend = a;
        end
    end
else
    Vc(1).xstart = [];
    Vc(1).xend = [];
    Vc(1).ystart = [];
    Vc(1).yend = [];
end

%Consolidate H, V, Hc and Vc into a set of line intersections. Include box
%boundary. A row of L has the form [ x0, y0, x1, y1 ] defining the
%intersection between (x0,y0) and (x1,y1)

%Store line elements
L = [0,0,1,0 ;...
    1,0,1,a ;...
    0,a,1,a;...
    0,0,0,a ];
for k=1:length(H)
    L = [L; H(k).xstart, H(k).ystart,  H(k).xend,  H(k).yend];
end
for k=1:length(V)
    L = [L; V(k).xstart, V(k).ystart, V(k).xend,  V(k).yend];
end
for k=1:length(Hc)
    L = [L; Hc(k).xstart, Hc(k).ystart, Hc(k).xend, Hc(k).yend];
end
for k=1:length(Vc)
    L = [L; Vc(k).xstart, Vc(k).ystart, Vc(k).xend, Vc(k).yend];
end

%Find intersection coordinates
out = lineSegmentIntersect(L,L);
x = out.intMatrixX; x = reshape( x, [ numel(x), 1 ] );
y = out.intMatrixY; y = reshape( y, [ numel(y), 1 ] );

%Get rid of NaN elements
nans = find( isnan(x) );
x( nans ) = [];
y( nans ) = [];

%Get rid of duplicates
r = unique( [x,y], 'rows' );
x = r(:,1);
y = r(:,2);

%To find the rectangles:
c = 1;
for m=1:M
    
    try
        % Choose a line intersection at random
        r = randi( length(x), 1 );
        x0 = x(r);
        y0 = y(r);
        
        % Find other intersections which have the same y coordinate
        s = find( y == y0 );
        xx = x(s);
        
        % Find the nearest of these and store the x coordinate
        [dist,i,j] = unique( abs( xx - x0 ) );
        x1 = xx( i(2) );
        
        % Find other intersections which have the same x coordinate as this
        s = find( x == x1 );
        yy = y(s);
        [dist,i,j] = unique( abs( yy - y0 ) );
        
        % Find the nearest of these and store the y coordinate
        y1 = yy( i(2) );
        
        % Use the coordinates above to create a rectangle
        R(m).x0 = x0;
        R(m).x1 = x1;
        R(m).y0 = y0;
        R(m).y1 = y1;
    catch
        
        %Typical error is a line intersection may occasionally not have more than one
        %intersections at the same x or y coordinate
        R(m).x0 = [];
        R(m).x1 = [];
        R(m).y0 = [];
        R(m).y1 = [];
        
    end
    
    %Cycle through rectangle colours
    if c==1
        R(m).colour = red;
    elseif c == 2
        R(m).colour = yellow;
    else
        R(m).colour = blue;
    end
    
    %Increment colour picker
    c = c + 1;
    if c > 3;
        c = 1;
    end
end

%Create a figure
figure( 'color', [ 1 1 1], 'name', 'Mondrian - by Andy French' )
axes('Nextplot','Add');
axis equal
axis off;

%Plot rectangles
for m=1:M
    patch( 'xdata',[ R(m).x0, R(m).x1, R(m).x1, R(m).x0 ],...
        'ydata',[R(m).y0, R(m).y0, R(m).y1, R(m).y1],...
        'Facecolor', R(m).colour, 'Edgecolor', R(m).colour );
end

%Plot lines
for k=1:length(H)
    plot( [ H(k).xstart, H(k).xend ], [ H(k).ystart, H(k).yend ], 'k', 'linewidth', t );
end
for k=1:length(V)
    plot( [ V(k).xstart, V(k).xend ], [ V(k).ystart, V(k).yend ], 'k', 'linewidth', t );
end
for k=1:length(Hc)
    plot( [ Hc(k).xstart, Hc(k).xend ], [ Hc(k).ystart, Hc(k).yend ], 'k', 'linewidth', t );
end
for k=1:length(Vc)
    plot( [ Vc(k).xstart, Vc(k).xend ], [ Vc(k).ystart, Vc(k).yend ], 'k', 'linewidth', t );
end

%plot(x,y,'m*')

%Print PNG and close figure
filename = ['mondrian ',strrep( datestr(now), ':','-' ),'.png'];
print( gcf, filename ,'-dpng','-r300' );
close(gcf);
croppic( filename, 255*[1 1 1] )

%%

% Function which divides up the interval [a,b] into (at most N) randomly
% sized parts, such that the smallest part is greater than d
function x = random_cut( a, b, N, d )

%Generate N random spacings of the interval [a,b]. Normalize the spacings
%such that their sum is b - a. Ignore spacings less than d
s = rand( 1, N );
s = (b-a)*s/sum(s);
s( s < d ) = [];

%Generate cut vector x from spacings
if isempty(s)
    %No selected spacings so cut [a,b] in half
    x = (a+b)/2;
else
    %Use random spacings to cut up [a,b]
    xx = a;
    x = [];
    for n=1:numel(s)
        xx = xx + s(n);
        x = [x,xx];
    end
end

%%

% Function which randomly splits a vector of values into two parts
function [x1,x2] = random_split( x )
n = randi( numel(x), 1 );
x1 = x( 1 : n );
if n==numel(x)
    x2 = [];
else
    x2 = x( n+1 : end );
end

%%

%croppic
% Automatic cropping of a bitmap image. This function determines the
% largest rectangular bounding box around pixels in the image which are NOT
% the background colour. The latter is specified as a [R,G,B] vector with
% values 0.... 255 for red, green and blue.

function croppic( filename, background_colour )

%Load image
I = imread(filename);

%Determine maximum and minimum indices of pixels which are NOT the
%paper colour
[row_ind,col_ind] =...
    find( (( I(:,:,1)==background_colour(1) ) .*...
    ( I(:,:,2)==background_colour(2) ) .*...
    ( I(:,:,3)==background_colour(3) ))==0 );
min_row = min(row_ind);
max_row = max(row_ind);
min_col = min(col_ind);
max_col = max(col_ind);

%Crop image
I = I( min_row : max_row, min_col:max_col, : );

%Save image
imwrite( uint8(I) , filename );

%%

%LINESEGMENTINTERSECT Intersections of line segments.
%   OUT = LINESEGMENTINTERSECT(XY1,XY2) finds the 2D Cartesian Coordinates of
%   intersection points between the set of line segments given in XY1 and XY2.
%
%   XY1 and XY2 are N1x4 and N2x4 matrices. Rows correspond to line segments. 
%   Each row is of the form [x1 y1 x2 y2] where (x1,y1) is the start point and 
%   (x2,y2) is the end point of a line segment:
%
%                  Line Segment
%       o--------------------------------o
%       ^                                ^
%    (x1,y1)                          (x2,y2)
%
%   OUT is a structure with fields:
%
%   'intAdjacencyMatrix' : N1xN2 indicator matrix where the entry (i,j) is 1 if
%       line segments XY1(i,:) and XY2(j,:) intersect.
%
%   'intMatrixX' : N1xN2 matrix where the entry (i,j) is the X coordinate of the
%       intersection point between line segments XY1(i,:) and XY2(j,:).
%
%   'intMatrixY' : N1xN2 matrix where the entry (i,j) is the Y coordinate of the
%       intersection point between line segments XY1(i,:) and XY2(j,:).
%
%   'intNormalizedDistance1To2' : N1xN2 matrix where the (i,j) entry is the
%       normalized distance from the start point of line segment XY1(i,:) to the
%       intersection point with XY2(j,:).
%
%   'intNormalizedDistance2To1' : N1xN2 matrix where the (i,j) entry is the
%       normalized distance from the start point of line segment XY1(j,:) to the
%       intersection point with XY2(i,:).
%
%   'parAdjacencyMatrix' : N1xN2 indicator matrix where the (i,j) entry is 1 if
%       line segments XY1(i,:) and XY2(j,:) are parallel.
%
%   'coincAdjacencyMatrix' : N1xN2 indicator matrix where the (i,j) entry is 1 
%       if line segments XY1(i,:) and XY2(j,:) are coincident.

% Version: 1.00, April 03, 2010
% Version: 1.10, April 10, 2010
% Author:  U. Murat Erdem

% CHANGELOG:
%
% Ver. 1.00: 
%   -Initial release.
% 
% Ver. 1.10:
%   - Changed the input parameters. Now the function accepts two sets of line
%   segments. The intersection analysis is done between these sets and not in
%   the same set.
%   - Changed and added fields of the output. Now the analysis provides more
%   information about the intersections and line segments.
%   - Performance tweaks.

% I opted not to call this 'curve intersect' because it would be misleading
% unless you accept that curves are pairwise linear constructs.
% I tried to put emphasis on speed by vectorizing the code as much as possible.
% There should still be enough room to optimize the code but I left those out
% for the sake of clarity.
% The math behind is given in:
%   http://local.wasp.uwa.edu.au/~pbourke/geometry/lineline2d/
% If you really are interested in squeezing as much horse power as possible out
% of this code I would advise to remove the argument checks and tweak the
% creation of the OUT a little bit.

function out = lineSegmentIntersect(XY1,XY2)

% Argument check.

validateattributes(XY1,{'numeric'},{'2d','finite'});
validateattributes(XY2,{'numeric'},{'2d','finite'});

[n_rows_1,n_cols_1] = size(XY1);
[n_rows_2,n_cols_2] = size(XY2);

if n_cols_1 ~= 4 || n_cols_2 ~= 4
    error('Arguments must be a Nx4 matrices.');
end

%Prepare matrices for vectorized computation of line intersection points.

X1 = repmat(XY1(:,1),1,n_rows_2);
X2 = repmat(XY1(:,3),1,n_rows_2);
Y1 = repmat(XY1(:,2),1,n_rows_2);
Y2 = repmat(XY1(:,4),1,n_rows_2);

XY2 = XY2';

X3 = repmat(XY2(1,:),n_rows_1,1);
X4 = repmat(XY2(3,:),n_rows_1,1);
Y3 = repmat(XY2(2,:),n_rows_1,1);
Y4 = repmat(XY2(4,:),n_rows_1,1);

X4_X3 = (X4-X3);
Y1_Y3 = (Y1-Y3);
Y4_Y3 = (Y4-Y3);
X1_X3 = (X1-X3);
X2_X1 = (X2-X1);
Y2_Y1 = (Y2-Y1);

numerator_a = X4_X3 .* Y1_Y3 - Y4_Y3 .* X1_X3;
numerator_b = X2_X1 .* Y1_Y3 - Y2_Y1 .* X1_X3;
denominator = Y4_Y3 .* X2_X1 - X4_X3 .* Y2_Y1;

u_a = numerator_a ./ denominator;
u_b = numerator_b ./ denominator;

% Find the adjacency matrix A of intersecting lines.
INT_X = X1+X2_X1.*u_a;
INT_Y = Y1+Y2_Y1.*u_a;
INT_B = (u_a >= 0) & (u_a <= 1) & (u_b >= 0) & (u_b <= 1);
PAR_B = denominator == 0;
COINC_B = (numerator_a == 0 & numerator_b == 0 & PAR_B);

% Arrange output.
out.intAdjacencyMatrix = INT_B;
out.intMatrixX = INT_X .* INT_B;
out.intMatrixY = INT_Y .* INT_B;
out.intNormalizedDistance1To2 = u_a;
out.intNormalizedDistance2To1 = u_b;
out.parAdjacencyMatrix = PAR_B;
out.coincAdjacencyMatrix= COINC_B;

%End of code
