%polygon_stitch
% Curve stitches the interior of a regular polygon.
%
% LAST UPDATED by Andy French April 2012

function polygon_stitch

%Number of sides
N = 5;

%Number of stitches per side
S = 32;

%Output image DPI
DPI = 300;

%Define colormap string
cmap_str = 'prism';

%Plot polygon outline?
plot_poly_outline = 1;

%%

%Create output image name
name = [num2str(N),'-gon with ',num2str(S),' stitches per side'];

%Define coordinates of polygon
t = linspace(0,2*pi,N+1).';
xp = sin(t);
yp = cos(t);

%Construct cell array of line segments which define the edges of the
%polygon.
edgeX = cell(1,N);
edgeY = cell(1,N);
edgeZ = cell(1,N);
for n=1:N-1
    edgeX{n} =[xp(n);xp(n+1)];
    edgeY{n} =[yp(n);yp(n+1)];
    edgeZ{n} = [0,0];
end
edgeX{N} = [xp(N);xp(1)];
edgeY{N} = [yp(N);yp(1)];
edgeZ{N} = [0,0];

%Stitch edges and store these in cell arrays
stitchX = cell(1,N);
stitchY = cell(1,N);
stitchZ = cell(1,N);
for n=1:N-1
    [x,y,z] = curve_stitch( S, edgeX{n},edgeY{n},edgeZ{n},...
        edgeX{n+1},edgeY{n+1},edgeZ{n+1} );
    stitchX{n} = x;
    stitchY{n} = y;
    stitchZ{n} = z;
end
[x,y,z] = curve_stitch( S, edgeX{N},edgeY{N},edgeZ{N},...
    edgeX{1},edgeY{1},edgeZ{1} );
stitchX{N} = x;
stitchY{N} = y;
stitchZ{N} = z;

%Create figure
figure('color',[1 1 1],'name','Polygon stitch');

%Interpolate colormap to find RGB vectors of stitch colours
cmap = colormap(cmap_str);
dim = size(cmap);
RGB = cell(1,N);
ind = linspace(1,dim(1),N);
for n=1:N
    RGB{n} = [
        interp1( (1:dim(1)).', cmap(:,1), ind(n) ),...
        interp1( (1:dim(1)).', cmap(:,2), ind(n) ),...
        interp1( (1:dim(1)).', cmap(:,3), ind(n) )
        ];
end

%Plot curves and stitches
if plot_poly_outline==1
    plot(xp,yp,'k','linewidth',2);
end
hold on
for n=1:N
    plot3(stitchX{n},stitchY{n},stitchZ{n},'color',RGB{n},'linewidth',0.01);
end
axis equal
view(2)
xlabel('x')
ylabel('y')
zlabel('z')
title(name)
axis off

%Print PNG output
print(gcf,'-dpng',['-r',num2str(DPI)], name )

%%

%curve_stitch
% Function which 'stitches' two curves with straight line segments.
% Curves are defined by column vectors of X,Y,Z coordinates.
%
% LAST UPDATED by Andy French April 2016

function [x,y,z] = curve_stitch( N, X1,Y1,Z1, X2,Y2,Z2 )

%Mark curves into N equal length pieces
[x1,y1,z1] = curvemark( N, X1,Y1,Z1 );
[x2,y2,z2] = curvemark( N, X2,Y2,Z2 );

%Join marks by line segments
x = [];
y = [];
z = [];
for n=1:N
    x = [ x,[x1(n);x2(n)] ];
    y = [ y,[y1(n);y2(n)] ];
    z = [ z,[z1(n);z2(n)] ];
end

%%

%curvemark
% Function which determines the x,y,z coordinates of N equally spaced
% points along the length of a curve defined in linear line segments
% between coordniates stored in (equal dimension) vectors X,Y,Z.
%
% LAST UPDATED by Andy French April 2012
%
% Syntax: [x,y,z] = curvemark( N, X,Y,Z )
%
% N is the number of points. Must be an integer!
% X,Y,Z vectors define the curve to be marked. They are all column
% vectors of dimensions [ M,1 ]. Note plot3(X,Y,Z) would plot the curve.
%
% x,y,z are corresponding column vectors of coordinates corresponding to N equally
% spaced points along the length of the curve defined by X,Y,Z.

function [x,y,z] = curvemark( N, X,Y,Z )

%Step through curve X,Y,Z and cumulatively work out total line length
M = length(X);
L = zeros(M,1);
for m=2:M
    L(m) = L(m-1) + sqrt( (X(m)-X(m-1))^2 + (Y(m)-Y(m-1))^2 + (Z(m)-Z(m-1))^2 );
end

%Interpolate L in N equal steps
Li = interp1( (1:M).', L, linspace( 1, M , N).' );

%Initialize outputs
x = zeros(N,1);
y = zeros(N,1);
z = zeros(N,1);

%Step through Li and (i) find indices m, m+1 of L s.t. L(m) <= Li(n) <=
% L(m+1).
[m,Lm] = approxfind( L,Li );

%Determine coordinates x,y,z from curve X,Y,Z via interpolation
for n=1:N
    dL = Li(n) - Lm(n);
    if dL == 0
        x(n) = X( m(n) );
        y(n) = Y( m(n) );
        z(n) = Z( m(n) );
    elseif dL > 0
        f = ( Li(n) - Lm(n) ) / ( L( m(n) + 1 ) - L( m(n) ) )  ; 
        dX = X( m(n)+1 ) - X( m(n) );
        dY = Y( m(n)+1 ) - Y( m(n) );
        dZ = Z( m(n)+1 ) - Z( m(n) );
        x(n) = X( m(n) ) + f*dX ;
        y(n) = Y( m(n) ) + f*dY ;
        z(n) = Z( m(n) ) + f*dZ ;
    else
        f = ( Li(n) - Lm(n) ) / ( L( m(n) ) - L( m(n) - 1 ) )   ; 
        dX = X( m(n) ) - X( m(n) - 1 );
        dY = Y( m(n) ) - Y( m(n) - 1 );
        dZ = Z( m(n) ) - Z( m(n) - 1 );
        x(n) = X( m(n) ) + f*dX ;
        y(n) = Y( m(n) ) + f*dY ;
        z(n) = Z( m(n) ) + f*dZ ;
    end
end

%%

%approxfind
% Function which returns the index of a vector x whose value is closest to a
% given vector of values v.
%
% LAST UPDATED by Andy French April 2012.

function [ix,xv] = approxfind(x,v)
ix = ones(size(v));
xv = zeros(size(v));
for n=1:length(v)
    z=abs(x-v(n));
    iix=find(min(z)==z);
    if isempty(iix)
        ix(n) = NaN;
        xv(n) = NaN;
    else
        ix(n) = iix(1);
        xv(n) = x( ix(n) );
    end    
end

%%

function approxfind_test

N = 100;
x = linspace(0,2*pi,N);
y = sin(x);
xx = x + 2*rand(1,N);
[ix,xv] = approxfind(x,xx);
figure('color',[1 1 1],'name','approxfind test')
plot(x,y,'b')
hold on;
plot(xv,sin(xv),'r*');
xlabel('x')
ylabel('y')
title('approxfind test')

%%

function curvemark_test

N = 40;
M = 200;

%Define curve parameterically
t = linspace(0,2*pi,M) + rand(1,M);
t = sort(t);
X = cos(t);
Y = sin(t);
Z = t/3;

%Turn coordinates into column vectors to enable straightforward plotting
X = X.';
Y = Y.';
Z = Z.';

%Mark curve into N equal length pieces
[x,y,z] = curvemark( N, X,Y,Z );

%Plot marked curve
figure('color',[1 1 1],'name','curvemark test')
plot3(X,Y,Z,'b');
hold on
plot3(x,y,z,'r*');
axis vis3d
axis equal
grid on
xlabel('x')
ylabel('y')
zlabel('z')
title('Curvemark test')

%End of code