%volume
% Visualize a scalar field defined by a 3D matrix V defined at points
% contained within an identically sized set of cartesian matrices X,Y,Z.
% These must be of a 'plaid' form - i.e. equivalent to the output of
% meshgrid.
%
% A volume is visualized by plotting nested shells of volume iso-surfaces with
% varying transparency. The iso-surface values are provided in vector v.
%
% LAST UPDATED by Andy French April 2016

function volume

%Get scalar field data from MATLAB test function flow (flow from a nozzle)
[x,y,z,V] = flow;
vmax = max(max(max(V)));
vmin = min(min(min(V)));

%Define values to evaluate iso-surface
k = [0.1,0.5,0.7,0.9,0.95];
v = vmin + k*(vmax-vmin);
t = 0.1+0.9*exp(4*(k-1));

%Perform volume visualization
cax = [];
vol( x,y,z,V,v,t,cax );

%Set graph parameters
colorbar
grid on
xlabel('x')
ylabel('y')
zlabel('z')
title('Test of volume.m')
view(63,18);
axis vis3d
lighting phong
camlight
interp_colormap(500);

%%

function vol( x,y,z,V,v,t,cax )

%Point to current active figure, or create a new one if one doesn't exist.
fig = gcf;

%Make figure window white and set rendering option to OpenGL. Use ZBuffer
%if your graphics card doesn't support this.
set(fig,'color',[1 1 1],'renderer','opengl');

%Obtain maximum and minimum values of V
Vmax = max(max(max(V)));
Vmin = min(min(min(V)));

%Make sure v is within the range Vmin to Vmax
v(v>Vmax) = Vmax;
v(v<Vmin) = Vmin;

%Make sure t is within the range 0 to 1
t(t>1) = 1;
t(t<0) = 0;

%Set limits of V to correspond to colormap
if isempty(cax)==1
    caxis( [Vmin,Vmax] );
else
    caxis( cax );
end
    
%Visualize volume by plotting nested shells of volume iso-surfaces with
%varying transparency. Use current colormap to set the colour of the shells
for n=1:length(v)
    [R,G,B] = colfromcolormap( v(n) );
    p = patch( isosurface(x,y,z,V, v(n) ),...
        'Facealpha',t(n),'FaceColor', [R,G,B], 'EdgeColor', 'none');
    isonormals(x,y,z,V,p);
end
view(3); 
daspect([1 1 1]);

%%

%colfromcolormap
% Obtain colours corresponding to value x from curent colormap.

function [R,G,B] = colfromcolormap(x)

%Get vaiable range of current colormap
[xmin,xmax] = caxis;

%Get current colormap
map = colormap;
dim = size(map);

%Make sure x is within the caxis range
x(x>xmax) = xmax;
x(x<xmin) = xmin;

%Interpolate R,G,B values at x
R = interp1( linspace(xmin,xmax,dim(1)),map(:,1).',x );
G = interp1( linspace(xmin,xmax,dim(1)),map(:,2).',x );
B = interp1( linspace(xmin,xmax,dim(1)),map(:,3).',x );

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code