%import_NUBASE
% Imports the NUBASE radioactive nuclide database into a matlab .mat file.
% This can then be used for analysis.
%
% The data is imported from an Excel sheet NUBASE, which has been
% constructed from the ASCII file which can be downloaded from:
%
% http://amdc.impcas.ac.cn/evaluation/data2012/nubase.html
%
% LAST UPDATED by Andy French March 2016

function import_NUBASE

%Number of nuclides in database
N = 5510;

%Mass of a proton /kg
mp = 1.672621778e-27;

%Mass of a neutron /kg
mn = 1.674927471e-27;

%Atomic mass unit /kg
u = 1.660539040e-27;

%Speed of light /ms^-1
c = 299792458;

%Charge of the electron /coulombs
qe = 1.6021766209e-19;

%Start clock and display message to command window
t0=tic;
disp(' ')
disp(' Importing nuclide data from Excel file NUBASE.xls into file nuclides.mat.' )
disp(' An Excel file better_NUBASE.xls will also be created, with columns of data' )
disp(' such as half-life, in the same units and free of text characters!' )
disp(' ')

%Load elements data elements.mat
d = load('elements.mat');

%Load NUBASE Excel file
[num,txt,raw] = xlsread('NUBASE.xls');

%%

%Initialize output vectors
Z = zeros(1,N);
A = zeros(1,N);
nuclide = cell(1,N);
B_MeV = zeros(1,N);
B_std_MeV = zeros(1,N);
E_excite_keV = zeros(1,N);
E_excite_std_keV = zeros(1,N);
half_life_s = zeros(1,N);
half_life_std_s = zeros(1,N);
stable_nuclei = zeros(1,N);
spin_and_parity = cell(1,N);
year_of_discovery = zeros(1,N);
reference = cell(1,N);
decay_mode_and_intensity = cell(1,N);

%Step through database and determine outputs
for n=1:N
    
    %Mass number
    A(n) = raw{n+1,2};
    
    %Nuclide name e.g. 1H, 8He. Strip off A prefix to help find Z
    nuclide{n} = raw{n+1,4};
    symbol = strrep(nuclide{n},num2str(A(n)),'');
    
    %Determine atomic number (Z) by cross referencing against
    %d.element_symbol and d.z. Some nuclides in NUBASE have up to two extra
    %characters e.g. 6Lii (i.e. 6 Li (i) ) or 10Bxi. Try removing these and
    %endeavour to match again. This problem would not have manifested if
    %the NUBASE people had bothered to include an Atomic Number column!
    i = strcmp( symbol, d.element_symbol );
    if sum(i)==0
        
        %Remove last character of string ....
        i = strcmp( symbol(1:end-1), d.element_symbol );
        if sum(i)==0
            
            %Remove next last character of string ...
            i = strcmp( symbol(1:end-2), d.element_symbol );
            if sum(i)==0
                %Give up if still no match
                Z(n) = NaN;
            else
                i = find( i==1 );
                i = i(1);
                Z(n) = d.Z(i(1));
            end
        else
            i = find( i==1 );
            i = i(1);
            Z(n) = d.Z(i(1));
        end
    else
        i = find( i==1 );
        i = i(1);
        Z(n) = d.Z(i(1));
    end
    
    %Binding energy /MeV, and standard error in this measurement /keV
    %The corresponding column in NUBASE is defined as the 'mass excess' (M - A) in keV
    %Carbon-12 is set at zero. A negative mass excess means a positive
    %binding energy, i.e. the mass of the nucleus is less than the
    %component parts. This means energy must be released if the nucleus
    %decomposes into protons and neutrons.
    M_MeV = data2num( raw{n+1,5} )/1000 + A(n)*u*(c^2)/(1e6*qe);
    B_MeV(n) = ( A(n) - Z(n) )*mn*(c^2)/(1e6*qe) + Z(n)*mp*(c^2)/(1e6*qe) - M_MeV;
    B_std_MeV(n) = data2num( raw{n+1,6} )/1000;
    
    %Excitation energy /MeV, and standard error in this measurement /keV
    E_excite_keV(n) = data2num( raw{n+1,7} );
    E_excite_std_keV(n) =data2num( raw{n+1,8} ) ;
    
    %Half life (numeric) and uncertainty (numeric)
    half_life =raw{n+1,10} ;
    half_life_std =raw{n+1,12} ;
    
    %Half life time unit
    time_unit = raw{n+1,11};
    
    %Half life, and half life uncertainty /s
    half_life = convert_half_life( half_life, time_unit );
    half_life_s(n) = half_life;
    half_life_std = convert_half_life( half_life_std, time_unit );
    half_life_std_s(n) = half_life_std;
    
    %Is nuclei stable?
    if strcmp( raw{n+1,10}, 'stbl')
        stable_nuclei(n) = 1;
    end
    
    %Spin & parity
    spin_and_parity{n} = raw{n+1,13};
    
    %Source reference string
    reference{n} = raw{n+1,15};
    
    %Year of discovery
    year_of_discovery(n) = raw{n+1,16};
    
    %Decay mode and intensities (%)
    decay_mode_and_intensity{n} = raw{n+1,17};
end

%Determie binding energy per nucleon
B_MeV_per_nucleon = B_MeV./A;

%Save .mat file
clear num txt raw n i time_unit half_life half_life_std symbol d
save nubase
disp([' Data imported and saved to file nubase.mat in ',num2str(toc(t0)),'s.']);
disp(' ');

%Construct Excel file better_NUBASE.xls and save this
disp(' Constructing better NUBASE Excel sheet ...')
data = cell(N+1,15);
data{1,1} = 'Atomic number Z';
data{1,2} = 'Mass number A';
data{1,3} = 'Nuclide';
data{1,4} = 'Binding energy /MeV';
data{1,5} = 'Binding energy error /MeV';
data{1,6} = 'Binding energy per nucleon /MeV';
data{1,7} = 'Excitation energy /keV';
data{1,8} = 'Excitation energy error /keV';
data{1,9} = 'half life /s';
data{1,10} = 'half life error /s';
data{1,11} = 'Is nuclei stable?';
data{1,12} = 'Spin & Parity';
data{1,13} = 'Year of discovery';
data{1,14} = 'NUBASE Reference';
data{1,15} = 'Decay mode & intensity';
for n=1:N
    data{n+1,1} = Z(n);
    data{n+1,2} = A(n);
    data{n+1,3} = nuclide{n};
    data{n+1,4} = B_MeV(n);
    data{n+1,5} = B_std_MeV(n);
    data{n+1,6} = B_MeV_per_nucleon(n);
    data{n+1,7} = E_excite_keV(n);
    data{n+1,8} = E_excite_std_keV(n);
    data{n+1,9} = half_life_s(n);
    data{n+1,10} = half_life_std_s(n);
    data{n+1,11} = stable_nuclei(n);
    data{n+1,12} = spin_and_parity{n};
    data{n+1,13} = year_of_discovery(n);
    data{n+1,14} = reference{n};
    data{n+1,15} = decay_mode_and_intensity{n};
end
xlswrite('better_NUBASE.xls',data);
disp(' better_NUBASE.xls created.')
disp(' ')

%%

% Remove extra text appendages from numeric data
function y = data2num(x)
y = x;
if ischar(x)
    x = strrep(x,'>','');
    x = strrep(x,'<','');
    x = strrep(x,'#','');
    x = str2num(x);
end
if isempty(x)
    %x is still not a number, hence NaN output
    y = NaN;
else
    %So x is definitely a number, but could be two values!
    y = x(1);
end

%%

% Convert half-life data into half-life /s
function t = convert_half_life( half_life, time_unit )

%If half life string means 'stable' set half life to be infinite
if strcmp( half_life, 'stbl' )
    t = inf;
    
else
    %Convert to a number, stripping off any text characters like #, < etc
    t = data2num(half_life);
end

%Work out time multiplier
if strcmp( time_unit, 's' )
    m = 1;
elseif strcmp( time_unit, 'm' )
    m = 60;
elseif strcmp( time_unit, 'h' )
    m = 3600;
elseif strcmp( time_unit, 'd' )
    m = 3600*24;
elseif strcmp( time_unit, 'y' )
    m = 3600*24*365.2422;
elseif strcmp( time_unit, 'ms' )
    m = 1e-3;
elseif strcmp( time_unit, 'us' )
    m = 1e-6;
elseif strcmp( time_unit, 'ns' )
    m = 1e-9;
elseif strcmp( time_unit, 'ps' )
    m = 1e-12;
elseif strcmp( time_unit, 'fs' )
    m = 1e-15;
elseif strcmp( time_unit, 'as' )
    m = 1e-18;
elseif strcmp( time_unit, 'zs' )
    m = 1e-21;
elseif strcmp( time_unit, 'ys' )
    m = 1e-24;
elseif strcmp( time_unit, 'ky' )
    m = 1e3*3600*24*365.2422;
elseif strcmp( time_unit, 'My' )
    m = 1e6*3600*24*365.2422;
elseif strcmp( time_unit, 'Gy' )
    m = 1e9*3600*24*365.2422;
elseif strcmp( time_unit, 'Ty' )
    m = 1e12*3600*24*365.2422;
elseif strcmp( time_unit, 'Py' )
    m = 1e15*3600*24*365.2422;
elseif strcmp( time_unit, 'Ey' )
    m = 1e18*3600*24*365.2422;
elseif strcmp( time_unit, 'Zy' )
    m = 1e21*3600*24*365.2422;
elseif strcmp( time_unit, 'Yy' )
    m = 1e24*3600*24*365.2422;
else
    m=1;
end

%Calculate time /s
t = t*m;

%End of code