%plot_binding_energy
% Function which loads nuclide data and plots Atomic number (Z) vs
% Mass number (A) for various isotopes, colour coded
% by binding energy /MeV.
%
% LAST UPDATED by Andy French March 2016

function plot_binding_energy

%Set maximum Z and A-Z values for graph
Zmax = 120;
Nmax = 180;

%Define Binding Energy scale /MeV
B_scale = [0,2200];
B_scale_per_nucleon = [6,8.5];

%Fontsize for plots
fsize = 16;

%

%Determine Z, A and half_life_s data
d = load('nubase.mat');
Z = d.Z;
A = d.A;
B_MeV = d.B_MeV;
B_MeV_per_nucleon = d.B_MeV_per_nucleon;
N = length(Z);

%Create figure
figure('name','Binding energy /MeV of radioactive isotopes','color',[1 1 1]);

%Determine colour map
cmap = colormap;
interp_colormap(500);

%Determine red, green and blue colours appropriate to each binding energy
red = cmap(:,1);
green = cmap(:,2);
blue = cmap(:,3);
R = zeros(1,N);
G = zeros(1,N);
B = zeros(1,N);
plot_ok=ones(1,N);
for n=1:N
    x = B_MeV(n);
    
    if isnan(x)==0
        %'Round up' any half lives that are less than colour scale
        if x <  B_scale(1)
            x =  B_scale(1);
        elseif x > B_scale(2)
            x =  B_scale(2);
        end
        
        %Interpolate colour map to determine plot colour
        R(n) = interp1( linspace( B_scale(1), B_scale(2), length(red) ), red.',x );
        G(n) = interp1( linspace( B_scale(1), B_scale(2), length(red) ), green.', x);
        B(n) = interp1( linspace( B_scale(1), B_scale(2), length(red) ), blue.', x);
    else
        plot_ok(n)=0;
    end
end

%

%Plot binding energy
hold on;
for n=1:N
    if plot_ok(n)==1
        plot(Z(n),A(n)-Z(n),'s','markeredgecolor',[R(n),G(n),B(n)],...
            'markerfacecolor',[R(n),G(n),B(n)],'markersize',1);
    end
end
xlabel('Atomic number Z','fontsize',fsize)
ylabel('Number of neutrons (A-Z)','fontsize',fsize)
title('Binding energy /MeV for isotopes','fontsize',fsize)
grid on
caxis(B_scale)
axis equal
xlim([0,Zmax]);
ylim([0,Nmax]);
plot( linspace(0,Zmax,N), linspace(0,Zmax,N), 'k' );
colorbar('fontsize',fsize)
set(gca,'fontsize',fsize)
box on
print( gcf,'isotope binding energy.png','-dpng','-r300');
cla;

%

%Determine red, green and blue colours appropriate to each binding energy
%per nucleon
for n=1:N
    %'Round up' any half lives that are less than colour scale
    x = B_MeV_per_nucleon(n);
    
    if isnan(x)==0
        if x <  B_scale_per_nucleon(1)
            x =  B_scale_per_nucleon(1);
        elseif x > B_scale_per_nucleon(2)
            x =  B_scale_per_nucleon(2);
        end
        
        %Interpolate colour map to determine plot colour
        R(n) = interp1( linspace( B_scale_per_nucleon(1), B_scale_per_nucleon(2), length(red) ), red.',x );
        G(n) = interp1( linspace( B_scale_per_nucleon(1), B_scale_per_nucleon(2), length(red) ), green.', x);
        B(n) = interp1( linspace( B_scale_per_nucleon(1), B_scale_per_nucleon(2), length(red) ), blue.', x);
    else
        plot_ok(n)=0;
    end
end

%

%Plot binding energy per nucleon
hold on;
for n=1:N
    if plot_ok(n)==1
        plot(Z(n),A(n)-Z(n),'s','markeredgecolor',[R(n),G(n),B(n)],...
            'markerfacecolor',[R(n),G(n),B(n)],'markersize',1);
    end
end
xlabel('Atomic number Z','fontsize',fsize)
ylabel('Number of neutrons (A-Z)','fontsize',fsize)
title('Binding energy per nucleon /MeV for isotopes','fontsize',fsize)
grid on
caxis(B_scale_per_nucleon)
axis equal
xlim([0,Zmax]);
ylim([0,Nmax]);
plot( linspace(0,Zmax,N), linspace(0,Zmax,N), 'k' );
colorbar('fontsize',fsize)
set(gca,'fontsize',fsize)
box on
print( gcf,'isotope binding energy per nucleon.png','-dpng','-r300');
close(gcf);

%%

%interp_colormap
% Function which interpolates current colormap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code