%plot_binding_energy_liquid_drop
% Function which generates a surface of binding energy /MeV per nucleon
% using the 'liquid drop model', which is plotted against atomic number 
% (Z) and neutron number (A-Z)
%
% LAST UPDATED by Andy French March 2016

function plot_binding_energy_liquid_drop

%Set maximum Z and A-Z values for graph
Zmax = 120;
Nmax = 180;

%Number of Z and Z values within this range
P = 100;

%Define Binding Energy scale /MeV
B_scale = [0,9];

%Fontsize for plots
fsize = 16;

%

%Determine Z, N and B (MeV per nucleon) using the liquid drop model
N = linspace(0,Nmax,P);
Z = linspace(0,Zmax,P);
[Z,N] = meshgrid(Z,N);
B = liquid_drop_model( N,Z );

%

%Create figure
figure('name','Binding energy /MeV of radioactive isotopes','color',[1 1 1]);

%Determine colour map
cmap = colormap;
interp_colormap(500);

%Plot binding energy per nucleon - liquid drop model
surf(Z,N,B);
shading interp
view(2)
xlabel('Atomic number Z','fontsize',fsize)
ylabel('Number of neutrons (A-Z)','fontsize',fsize)
title('Binding energy per nucleon /MeV for isotopes','fontsize',fsize)
grid on
caxis(B_scale)
axis equal
xlim([0,Zmax]);
ylim([0,Nmax]);
colorbar('fontsize',fsize)
set(gca,'fontsize',fsize)
box on
print( gcf,'isotope binding energy per nucleon - liquid drop.png','-dpng','-r300');
close(gcf);

%%

%Binding energy /MeV per nucleon as a function of Neutron number (N) and Proton number (Z)
% https://en.wikipedia.org/wiki/Semi-empirical_mass_formula
% As the name suggests, it is based partly on theory and partly on empirical
% measurements. The theory is based on the liquid drop model proposed by
% George Gamow, which can account for most of the terms in the formula and
% gives rough estimates for the values of the coefficients. It was first
% formulated in 1935 by German physicist Carl Friedrich von Weizscker.
function B = liquid_drop_model( N,Z )

%Constants /MeV
a_v = 15.8;
a_s = 18.3;
a_c = 0.714;
a_a = 23.2;
a_p = 12;

%Mass number
A = Z + N;

%Pairing term
dA = zeros(size(A));
both_even = find(( rem(Z,2)==0 ) & ( rem(N,2)==0 ) );
both_odd = find(( rem(Z,2)~=0 ) & ( rem(N,2)~=0 ) );
dA(both_even) = a_p*A(both_even).^(-3/4);
dA(both_odd) = -a_p*A(both_odd).^(-3/4);

%Compute Binding Energy per nucleon /MeV
B = a_v*A - a_s*A.^(2/3) -a_c*(Z.^2)./(A.^(1/3)) - a_a*((N-Z).^2)./A + dA;
B = B./A;

%interp_colormap
% Function which interpolates current colormap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code