%plot_isotope_half_life
% Function which loads nuclide data and plots
% Atomic number (Z) vs Mass number (A) for various isotopes, colour coded
% by half life. A logarithmic scale for the latter is used.
%
% LAST UPDATED by Andy French March 2016

function plot_isotope_half_life

%Set database (default is the wikipedia table)
database = 'nubase';

%Define colour scale for half lives (based upon power of ten of time in s)
half_life_scale = [-10,15];

%Set maximum Z and A-Z values for graph
Zmax = 120;
Nmax = 180;

%Define fontsize for graph
fsize = 16;

%

%Determine Z, A and half_life_s data
if strcmp(database,'nubase')
    d = load('nubase.mat');
else
    d = load('isotope_half_lives.mat');
end
Z = d.Z;
A = d.A;
half_life_s = d.half_life_s;
N = length(Z);

%Create figure
figure('name','half life of radioactive isotopes','color',[1 1 1]);

%Determine colour map
cmap = colormap;
interp_colormap(500);

%Determine red, green and blue colours appropriate to each half life
red = cmap(:,1);
green = cmap(:,2);
blue = cmap(:,3);
for n=1:N
    
    %Determine log10 of half life, and clip to colour scale
    x = log10(half_life_s(n));
    
    if x >  half_life_scale(2)
        %Black colour for stable nuclei
        R(n) = 0;
        G(n) = 0;
        B(n) = 0;
    else
        %'Round up' any half lives that are less than colour scale
        if x <  half_life_scale(1)
            x =  half_life_scale(1);
        end
        
        %Interpolate colour map to determine plot colour
        R(n) = interp1( linspace( half_life_scale(1), half_life_scale(2), length(red) ), red.', x);
        G(n) = interp1( linspace( half_life_scale(1), half_life_scale(2), length(red) ), green.', x);
        B(n) = interp1( linspace( half_life_scale(1), half_life_scale(2), length(red) ), blue.', x);
    end
end

%

%Plot data
hold on;
for n=1:N
    if ~isnan(R(n))
        plot(Z(n),A(n)-Z(n),'s','markeredgecolor',[R(n),G(n),B(n)],...
            'markerfacecolor',[R(n),G(n),B(n)],'markersize',1);
    end
end
xlabel('Atomic number Z','fontsize',fsize)
ylabel('Number of neutrons (A-Z)','fontsize',fsize)
title('log_{10}(half life /s) for isotopes','fontsize',fsize)
grid on
caxis(half_life_scale)
axis equal
xlim([0,Zmax]);
ylim([0,Nmax]);
plot( linspace(0,Zmax,N), linspace(0,Zmax,N), 'k' );
colorbar('fontsize',fsize)
set(gca,'fontsize',fsize)
box on
print( gcf,'isotope half lives.png','-dpng','-r300');
close(gcf);

%%

%interp_colormap
% Function which interpolates current colormap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code