%plot_year_of_discovery
% Function which loads nuclide data and plots Atomic number (Z) vs
% Mass number (A) for various isotopes, colour coded by year of discovery.
%
% LAST UPDATED by Andy French March 2016

function plot_year_of_discovery

%Set maximum Z and A-Z values for graph
Zmax = 120;
Nmax = 180;

%Define year of discovery scale
scale = [1940,2010];

%Fontsize for plots
fsize = 16;

%

%Determine Z, A and half_life_s data
d = load('nubase.mat');
y = d.year_of_discovery;
N = length(d.Z);

%Create figure
figure('name','year of discovery of nuclides','color',[1 1 1]);

%Determine colour map
cmap = colormap;
interp_colormap(500);

%Determine red, green and blue colours appropriate to each binding energy
red = cmap(:,1);
green = cmap(:,2);
blue = cmap(:,3);
R = zeros(1,N);
G = zeros(1,N);
B = zeros(1,N);
plot_ok = ones(1,N);
for n=1:N
    x = y(n);
    if ( d.stable_nuclei(n)==1 )
        %Set stable element values (i.e. 'ancient' elements with no fixed 'discovery'
        %date, or elements which have possibly not been discovered yet) to
        %be plotted as white
        R(n) = 1;
        G(n) = 1;
        B(n) = 1;
    elseif isnan(x)==1
        plot_ok(n)=0;
    else
            %'Round up' any half lives that are less than colour scale
        if x <  scale(1)
            x =  scale(1);
        elseif x > scale(2)
            x =  scale(2);
        end
        
        %Interpolate colour map to determine plot colour
        R(n) = interp1( linspace( scale(1), scale(2), length(red) ), red.',x );
        G(n) = interp1( linspace( scale(1), scale(2), length(red) ), green.', x);
        B(n) = interp1( linspace( scale(1), scale(2), length(red) ), blue.', x);
    end
end

%

%Plot year of discovery
hold on;
for n=1:N
    if plot_ok(n)==1
        plot(d.Z(n),d.A(n)-d.Z(n),'s','markeredgecolor',[R(n),G(n),B(n)],...
            'markerfacecolor',[R(n),G(n),B(n)],'markersize',1);
    end
end
xlabel('Atomic number Z','fontsize',fsize)
ylabel('Number of neutrons (A-Z)','fontsize',fsize)
title('Year of discovery of nuclides','fontsize',fsize)
grid on
caxis(scale)
axis equal
xlim([0,Zmax]);
ylim([0,Nmax]);
plot( linspace(0,Zmax,N), linspace(0,Zmax,N), 'k' );
colorbar('fontsize',fsize)
set(gca,'fontsize',fsize)
box on
print( gcf,'year of discovery.png','-dpng','-r300');
close(gcf);

%%

%interp_colormap
% Function which interpolates current colormap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code