%bounce
%
% Simulation where a partcle is created with a random initial velocity
% inside a 2D space. Inside is a randomly generated polygon. The particle
% will collide elastically with the edges of the box, and the polygon.
%
% LAST UPDATED by Andy French May 2016

function bounce

%Create box
fig = figure('color',[ 1 1 1],'name','bounce' );
ax = axes;

%Place polygon in the box
N = 20;
[verts_x,verts_y,vertices,edges] = create_random_poly(N);
patch(verts_x,verts_y,'g');
hold on;
axis equal
axis manual
box on

%Determine limits of box
xlimits = 2*get(gca,'xlim');
ylimits = 2*get(gca,'ylim');
set( gca, 'xlim', xlimits ,'ylim',ylimits );

%Initialize particle position and velocity
x = xlimits(1) + rand(1,1)*( xlimits(2) - xlimits(1) );
y = xlimits(1) + rand(1,1)*( ylimits(2) - ylimits(1) );
angle = 2*pi*rand(1,1);
vx = 20*cos(angle);
vy = 20*sin(angle);

%Plot partcle
p = plot( x,y,'r*' );

%Animate particle
t0 = tic;
in_poly = 0;
go = 1;
while ( go == 1 ) && ( ishandle(fig) == 1 )
    
    %Stop clock to work out time elapsed
    dt = toc(t0);
    
    %Update position of particle
    x = x + vx*dt;
    y = y + vy*dt;
    set( p, 'xdata', x,'ydata', y );
    drawnow;
    
    %Check if any collision has occured with a polygon edge
    [ vx,vy,in_poly ] = bouncecheck( in_poly, x,y, vx, vy, verts_x, verts_y, vertices, edges );
    
    %Check if particle has collided with the walls
    if ( x < xlimits(1) ) && (vx < 0 )
        vx = -vx;
    end
    if ( x > xlimits(2) ) && (vx > 0 )
        vx = -vx;
    end
    if ( y < ylimits(1) ) && (vy < 0 )
        vy = -vy;
    end
    if ( y > ylimits(2) ) && (vy > 0 )
        vy = -vy;
    end
       
    %Restart clock
    t0 = tic;
    pause(0.01);
    
end

%%

%Check whether x,y coordinates are in a polygon (or not)
%
% ISIN = ISINPOLY(X,Y,XP,YP)   Finds whether points with
%           coordinates X and Y are inside or outside of
%           a polygon with vertices XP, YP.
%           Returns matrix ISIN of the same size as X and Y
%           with 0 for points outside a polygon, 1 for
%           inside points and  0.5 for points belonging
%           to a polygon XP, YP itself.
%
%   By Kirill K. Pankratov
%	kirill@plume.mit.edu
%	4/10/94, 8/26/94.
function  isin = isinpoly(x,y,xp,yp)
if nargin<4
  fprintf('\n  Error: not enough input arguments.\n\n')
  return
end

 % Make the contour closed and get the sizes
xp = [xp(:); xp(1)];
yp = [yp(:); yp(1)];
sz = size(x);
x = x(:); y = y(:);

lp = length(xp); l = length(x);
ep = ones(1,lp); e = ones(1,l);

 % Calculate cumulative change in azimuth from points x,y
 % to all vertices
A = diff(atan2(yp(:,e)-y(:,ep)',xp(:,e)-x(:,ep)'))/pi;
A = A+2*((A<-1)-(A>1));
isin = any(A==1)-any(A==-1);
isin = (abs(sum(A))-isin)/2;

 % Check for boundary points
A = (yp(:,e)==y(:,ep)')&(xp(:,e)==x(:,ep)');
fnd = find(any(A));
isin(fnd) = .5*ones(size(fnd));
isin = round(isin*2)/2;

 % Reshape output to the input size
isin = reshape(isin,sz(1),sz(2));

%%

%distance
% Function which computes Euclidean distance matrix.
% This fully vectorized (VERY FAST!) m-file computes the
% Euclidean distance between two vectors by:
%          ||A-B|| = sqrt ( ||A||^2 + ||B||^2 - 2*A.B )
%
% Syntax: E = distance(A,B)
%    A - (DxM) matrix
%    B - (DxN) matrix
%    E - (MxN) Euclidean distances between vectors in A and B
%
% Example :
%    A = rand(400,100); B = rand(400,200);
%    d = distance(A,B);

% Author   : Roland Bunschoten
%            University of Amsterdam
%            Intelligent Autonomous Systems (IAS) group
%            Kruislaan 403  1098 SJ Amsterdam
%            tel.(+31)20-5257524
%            bunschot@wins.uva.nl
% Last Rev : Oct 29 16:35:48 MET DST 1999
% Tested   : PC Matlab v5.2 and Solaris Matlab v5.3
% Thanx    : Nikos Vlassis

function d = distance(a,b)
if (nargin ~= 2)
    error('Not enough input arguments');
end
if (size(a,1) ~= size(b,1))
    error('A and B should be of same dimensionality');
end
aa=sum(a.*a,1); bb=sum(b.*b,1); ab=a'*b;
d = sqrt(abs(repmat(aa',[1 size(bb,2)]) + repmat(bb,[size(aa,2) 1]) - 2*ab));

%%

%create_random_poly
% Function which creates a random polygon of N vertices

function [x,y,vertices,edges] = create_random_poly(N)

%Create N angles in range 0,2*pi and sort is ascending order
theta = sort( 2*pi*rand(1,N) );

%Create N radii
R = rand(1,N) + 2;

%Create vertices
x = [];
y = [];
for n=1:N
    vertices(n).x = R(n)*sin(theta(n));
    vertices(n).y = R(n)*cos(theta(n));
    x = [x,vertices(n).x];
    y = [y,vertices(n).y];
end

%Create edges
for n=1:N-1
    edges(n).x1 = vertices(n).x;
    edges(n).y1 = vertices(n).y;
    edges(n).x2 = vertices(n+1).x;
    edges(n).y2 = vertices(n+1).y;
    edges(n).x_mid = 0.5*( edges(n).x1 + edges(n).x2 );
    edges(n).y_mid = 0.5*( edges(n).y1 + edges(n).y2 );
    edges(n).length = sqrt( ( edges(n).x2 - edges(n).x1 )^2 + ( edges(n).y2 - edges(n).y1 )^2 );
    edges(n).edge_vector = [ edges(n).x2 - edges(n).x1, edges(n).y2 - edges(n).y1 ];
    edges(n).normal = [ -edges(n).edge_vector(2), edges(n).edge_vector(1) ];
    edges(n).normal = edges(n).normal/sqrt(dot(edges(n).normal,edges(n).normal));
end
edges(N).x1 = vertices(N).x;
edges(N).y1 = vertices(N).y;
edges(N).x2 = vertices(1).x;
edges(N).y2 = vertices(1).y;
edges(N).x_mid = 0.5*( edges(N).x1 + edges(N).x2 );
edges(N).y_mid = 0.5*( edges(N).y1 + edges(N).y2 );
edges(N).length = sqrt( ( edges(N).x2 - edges(N).x1 )^2 + ( edges(N).y2 - edges(N).y1 )^2 );
edges(N).edge_vector = [ edges(N).x2 - edges(N).x1, edges(N).y2 - edges(N).y1 ];
edges(N).normal = [ edges(N).y1 - edges(N).y2, edges(N).x2 - edges(N).x1 ];
edges(N).normal = edges(n).normal/sqrt(dot(edges(N).normal,edges(N).normal));

%%

%bouncecheck
% This function checks whether a particle with x,y coordinates
% (and velocity vx,vy) is within a polygon with provided vertices and edges
% structures. If the particle is inside the polygon (and wasn't previously)
% then a bounce event is determined, and an impulse is applied to the
% particle to cause it to exit the polygon.
%
% LAST UPDATED by Andy French 7th November 2013

function [ vx,vy,in_poly ] = bouncecheck( in_poly, x,y, vx, vy, verts_x, verts_y, vertices, edges )

%Check if particle is inside polygon
isin = isinpoly(x,y,verts_x,verts_y);

if ( isin==0 ) && ( in_poly == 1 )
    %Particle has now moved out of the polygon
    in_poly = 0;
elseif ( isin ==1 ) && ( in_poly == 0 )
    %Particle is 'newly' inside the polygon, i.e. in_poly = 0 so it wasn't
    %in the previous time step. This means a collision with the nearest
    %edge.
    in_poly = 1;
    
    %Determine which edge the particle has just crossed. Step through each
    %edge and work out the distance from the current particle location to
    %the edge line. The chosen edge is the one corresponding to the
    %smallest distance.
    d = zeros(1,length(edges));
    for n=1:length( edges )
        a = [ edges(n).x1, edges(n).y1 ];
        b = edges(n).edge_vector;
        d_vec = ( a - [x,y] ) - b*dot( a - [x,y], b );
        d(n) = dot( d_vec, d_vec );       
    end
    edge_n = find( d == min(d) );
    
    %Apply impulse to particle, based upon normal to edge which has just
    %been crossed and current velocity
    dv = -2*dot( [vx,vy], edges( edge_n ).normal )* edges( edge_n ).normal;
    vx = vx + dv(1);
    vy = vy + dv(2);   
end

%End of code