%cell_life
% Simulation based on Conway's Game of Life (1970)
% http://en.wikipedia.org/wiki/Conways_Game_of_Life. The universe of the
% Game of Life is an infinite two-dimensional orthogonal grid of square
% cells, each of which is in one of two possible states, alive or dead.
% Every cell interacts with its eight neighbours, which are the cells that
% are horizontally, vertically, or diagonally adjacent. At each step in
% time, the following transitions occur:
%
% 1. Any live cell with fewer than
%    two live neighbours dies, as if caused by under-population.
%
% 2. Any live cell with two or three live neighbours lives on to the
%    next generation.
%
% 3. Any live cell with more than three live neighbours dies, as if by
%    overcrowding.
%
% 4. Any dead cell with exactly three live neighbours becomes a live cell,
%    as if by reproduction.
%
% LAST UPDATED by A. French April 2016
% Original code by A. French, T. Redfern and J. Ganendra 2013.

function cell_life

%Initialize global data structure d
global d

%Display starting message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%% ')
disp('  Cell Life Simulation ')
disp(' %%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Get a PNG image as a starting point (turned into black and white)
%or create a randomly populated array
option = input( ' PNG image input or randomly populated image? :   ','s');
disp(' ');

if strcmp( option, 'PNG' ) == 1
    %Find the PNG file
    [filename, pathname, filterindex] = uigetfile('*.png', 'Select PNG file');
    if filename ~= 0
        %Load the file
        I = imread( [pathname,'\',filename] );
        
        %Turn into Black and White Image
        dim = size(I);
        rows = dim(1);
        cols = dim(2);
        A = zeros(rows,cols);
        A(:,:) = I(:,:,1);   %Just take first (red) colour plane
        %A(:,:) = (1/3) * ( I(:,:,1) + I(:,:,2) + I(:,:,3) );
        clear I
        d.A = zeros(rows,cols);
        d.A( A < median(median(A) )) = 1;
        clear A;
    end
else
    %Create a random array of 0s and 1s
    
    %Get the array dimensions
    rows = input(' Number of rows                       :    ');
    if isempty(rows)
        rows = 100;
        disp(' Default number of rows = 100' );
    end
    disp(' ');
    cols = input(' Number of columns                    :    ');
    if isempty(cols)
        cols = 100;
        disp(' Default number of columns = 100' );
    end
    disp(' ');
    
    %Get the fractional population
    fpop = input(' Define % of cells initially alive    :    ');
    if isempty( fpop )
        fpop = 20;
        disp(' Default % of cells initially alive = 20% ' );
    end
    fpop = fpop/100;
    if ( ( fpop < 0 ) || ( fpop > 1) )
        disp('Bad input, randomizing initial population');
        disp('Randomizing...')
        pause(1);
        fpop = rand(1);
        disp( ['Population fraction = ', num2str(100*fpop), '%'] )
    end
    d.fpop = fpop;
    
    %Create initial (binary) array describing cell world
    d.A = zeros(rows,cols);
    
    %Work out the number of alive cells
    N = floor( d.fpop*rows*cols );
    alive_indices = randperm(rows*cols);
    alive_indices = alive_indices(1:N);
    d.A(alive_indices) = 1;
    
    %Add an extra line return for elegance
    disp(' ');
end

%

%Create a figure and plot the arrray
fig = figure('name','Cell Life','color',[1 1 1],'toolbar','none','menubar','none','KeyPressFcn',@keypressfunc);
cmap = [ 1, 1, 1;
    0, 0, 0];
colormap(cmap);
caxis([0,1]);
p = pcolor(d.A);
shading flat
axis off
box on

%Loop simulation
d.print = 0;
d.run = 1;
d.random = 0;
iter = 1;
while ( d.run == 1 ) && ( ishandle(fig) == 1 )
    
    %Pause loop to force screen to update
    pause(0.02);
    
    %Update iteration number
    iter = iter + 1;
    
    %Work the number of live and dead neighbouring cells, for all cells in
    %the matrix
    [num_alive, num_dead] = neighbours( repmat( (1:rows).', [1,cols] ), repmat( 1:cols, [rows,1] ), d.A );
    new_A = d.A;
    
    %%% APPLY GAME OF LIFE RULES %%%
    
    % 1.Any live cell with fewer than
    % two live neighbours dies, as if caused by under-population.
    new_A( find( (d.A==1) & (num_alive < 2) ) ) = 0;
    
    % 2.Any live cell with two or three live neighbours lives on to the next generation.
    new_A( find( (d.A==1) & ( (num_alive == 2) | (num_alive == 3) ) ) ) = 1;
    
    % 3.Any live cell with more than three live neighbours dies, as if by
    % overcrowding.
    new_A( find( (d.A==1) & (num_alive > 3) ) ) = 0;
    
    % 4.Any dead cell with exactly three live neighbours becomes
    % a live cell, as if by reproduction.
    new_A( find( (d.A==0) & (num_alive == 3) ) ) = 1;
    
    %Update plot
    d.A = new_A;
    if ishandle(fig) == 1
        set( p, 'CData', d.A );
        drawnow;
    end
    
    %Print current view if 'p' key is pressed
    if d.print == 1
        print( fig, '-dpng','-r300',['cell_life. iter=',num2str(iter),...
            '. ',strrep(datestr(now),':','-'),'.png'])
        d.print = 0;
    end
    
    %Randomly make 0.25 x d.fpop cells alive if 'r' key is pressed
    if d.random == 1
        N = 0.25 * floor( d.fpop*rows*cols );
        alive_indices = randperm(rows*cols);
        alive_indices = alive_indices(1:N);
        d.A(alive_indices) = 1;
        d.random = 0;
    end
end

%%

%Function to characterize the neighbours around a given cell in a binary
%array. Alive cells ore ones, dead cells are zeros. Cell elements to be
%tested have row and column indices stored in matrices (which are of the
%same size) row and col.
function [num_alive, num_dead] = neighbours( row, col, A )

%Get dimensions of A
[num_rows,num_cols] = size(A);

%Initialize the outputs
num_alive = zeros(size(row));
num_dead = zeros(size(row));

%Work out cell indices of array A that are on the boundary, but not corner cells

%Left
i.left = find( (col==1) & (row~=1) & (row~=num_rows) );
num_alive(i.left) = num_alive(i.left) + A( sub2ind( size(A), row(i.left) - 1, col(i.left) ));     %Above
num_alive(i.left) = num_alive(i.left) + A( sub2ind( size(A), row(i.left) + 1, col(i.left) ));     %Below
num_alive(i.left) = num_alive(i.left) + A( sub2ind( size(A), row(i.left), col(i.left) + 1 ));     %Right
num_alive(i.left) = num_alive(i.left) + A( sub2ind( size(A), row(i.left)-1, col(i.left) + 1 ));   %Above right
num_alive(i.left) = num_alive(i.left) + A( sub2ind( size(A), row(i.left)+1, col(i.left) + 1 ));   %Below right
num_dead(i.left) = 5 - num_alive(i.left);

%Right
i.right = find( (col==num_cols) & (row~=1) & (row~=num_rows)  );
num_alive(i.right) = num_alive(i.right) + A( sub2ind( size(A), row(i.right) - 1, col(i.right) ));    %Above
num_alive(i.right) = num_alive(i.right) + A( sub2ind( size(A), row(i.right) + 1, col(i.right) ));    %Below
num_alive(i.right) = num_alive(i.right) + A( sub2ind( size(A), row(i.right), col(i.right) - 1 ));    %Left
num_alive(i.right) = num_alive(i.right) + A( sub2ind( size(A), row(i.right)-1, col(i.right) - 1 ));  %Above left
num_alive(i.right) = num_alive(i.right) + A( sub2ind( size(A), row(i.right)+1, col(i.right) - 1 ));  %Below left
num_dead(i.right) = 5 - num_alive(i.right);

%Top
i.top = find( (row==1) & (col~=1) & (col~=num_cols) );
num_alive(i.top) = num_alive(i.top) + A( sub2ind( size(A), row(i.top), col(i.top)-1 ));       %Left
num_alive(i.top) = num_alive(i.top) + A( sub2ind( size(A), row(i.top), col(i.top) + 1 ));     %Right
num_alive(i.top) = num_alive(i.top) + A( sub2ind( size(A), row(i.top) + 1, col(i.top) ));     %Below
num_alive(i.top) = num_alive(i.top) + A( sub2ind( size(A), row(i.top) + 1, col(i.top)-1 ));   %Below left
num_alive(i.top) = num_alive(i.top) + A( sub2ind( size(A), row(i.top) + 1, col(i.top)+1 ));   %Below right
num_dead(i.top) = 5 - num_alive(i.top);

%Bottom
i.bot = find( (row == num_rows) & (col~=1) & (col~=num_cols)  );
num_alive(i.bot) = num_alive(i.bot) + A( sub2ind( size(A), row(i.bot), col(i.bot) + 1 ));    %Left
num_alive(i.bot) = num_alive(i.bot) + A( sub2ind( size(A), row(i.bot), col(i.bot) - 1 ));    %Right
num_alive(i.bot) = num_alive(i.bot) + A( sub2ind( size(A), row(i.bot) - 1, col(i.bot) ));    %Above
num_alive(i.bot) = num_alive(i.bot) + A( sub2ind( size(A), row(i.bot)-1, col(i.bot) + 1 ));  %Above right
num_alive(i.bot) = num_alive(i.bot) + A( sub2ind( size(A), row(i.bot) - 1, col(i.bot)-1 ));  %Above left
num_dead(i.bot) = 5 - num_alive(i.bot);

%Top left
i.topleft = find( (row == 1) & (col == 1) );
num_alive(i.topleft) = num_alive(i.topleft) + A( sub2ind( size(A), row(i.topleft) + 1, col(i.topleft) ));    %Right
num_alive(i.topleft) = num_alive(i.topleft) + A( sub2ind( size(A), row(i.topleft) + 1, col(i.topleft) ));    %Below
num_alive(i.topleft) = num_alive(i.topleft) + A( sub2ind( size(A), row(i.topleft) + 1, col(i.topleft)+1 ));  %Top right
num_dead(i.topleft) = 3 - num_alive(i.topleft);

%Top right
i.topright = find( (row == 1) & (col == num_cols) );
num_alive(i.topright) = num_alive(i.topright) + A( sub2ind( size(A), row(i.topright), col(i.topright)-1 ));      %Left
num_alive(i.topright) = num_alive(i.topright) + A( sub2ind( size(A), row(i.topright) + 1, col(i.topright) ));    %Below
num_alive(i.topright) = num_alive(i.topright) + A( sub2ind( size(A), row(i.topright) + 1, col(i.topright)-1 ));  %Top left
num_dead(i.topright) = 3 - num_alive(i.topright);

%Bottom left
i.botleft = find( (row == num_rows) & (col == 1) );
num_alive(i.botleft) = num_alive(i.botleft) + A( sub2ind( size(A), row(i.botleft) - 1, col(i.botleft) ));    %Above
num_alive(i.botleft) = num_alive(i.botleft) + A( sub2ind( size(A), row(i.botleft), col(i.botleft)+1 ));      %Right
num_alive(i.botleft) = num_alive(i.botleft) + A( sub2ind( size(A), row(i.botleft)-1, col(i.botleft)+1 ));    %Bottom right
num_dead(i.botleft) = 3 - num_alive(i.botleft);

%Bottom right
i.botright = find( (row == num_rows) & (col == num_cols) );
num_alive(i.botright) = num_alive(i.botright) + A( sub2ind( size(A), row(i.botright) - 1, col(i.botright) ));  %Above
num_alive(i.botright) = num_alive(i.botright) + A( sub2ind( size(A), row(i.botright), col(i.botright) -1 ));   %Left
num_alive(i.botright) = num_alive(i.botright) + A( sub2ind( size(A), row(i.botright)-1, col(i.botright) -1 )); %Top left
num_dead(i.botright) = 3 - num_alive(i.botright);

%All other cells!
i.interior = find( (row>1) & (row<num_rows) & (col>1) & (col<num_cols) );
num_alive(i.interior) = num_alive(i.interior) + A( sub2ind( size(A), row(i.interior) - 1, col(i.interior) ));     %Above
num_alive(i.interior) = num_alive(i.interior) + A( sub2ind( size(A), row(i.interior) + 1, col(i.interior) ));     %Below
num_alive(i.interior) = num_alive(i.interior) + A( sub2ind( size(A), row(i.interior), col(i.interior) -1 ));      %Left
num_alive(i.interior) = num_alive(i.interior) + A( sub2ind( size(A), row(i.interior), col(i.interior) +1 ));      %Right
num_alive(i.interior) = num_alive(i.interior) + A( sub2ind( size(A), row(i.interior) - 1, col(i.interior)+1 ));   %Top right
num_alive(i.interior) = num_alive(i.interior) + A( sub2ind( size(A), row(i.interior) + 1, col(i.interior)+1 ));   %Bottom right
num_alive(i.interior) = num_alive(i.interior) + A( sub2ind( size(A), row(i.interior) -1, col(i.interior) -1 ));   %Top left
num_alive(i.interior) = num_alive(i.interior) + A( sub2ind( size(A), row(i.interior) +1, col(i.interior) -1 ));   %Bottom left
num_dead(i.interior) = 8 - num_alive(i.interior);

%%

% Key press handler
function keypressfunc( fig,evnt )
global d
if strcmp(get(gcf,'currentkey'),'q')==1
    d.run = 0;
    close(fig);
elseif strcmp(get(gcf,'currentkey'),'p')==1
    d.print = 1;
elseif strcmp(get(gcf,'currentkey'),'r')==1
    d.random = 1;
end

%End of code

