%conker
% Simulation of a conker being swung from a fixed point with no friction.
% Conker string is massless and inextensible. However it is NOT rigid!
% String when taut will bear ANY tension without breaking. When not taut,
% string bears no tension and the conker will move as a projectile. Air
% resistance is also ignored.
%
% The conker is initially held vertically and the given an impulse which
% causes it to have a rightwards initial velocity.
%
% Last updated by Andy French May 2016

function conker

%% DEFINE INPUTS %%

%Conker mass /kg
m = 0.05;

%String length /m
L = 0.2;

%Gravitational field strength /ms^-2
g = 9.81;

%Initial conker velocity /ms^-1
u = 0.8* sqrt( 5*g*L) ;

%Simulation time /s
tmax = 3 * 2*pi*L/u;

%Time interval /s between angle calculations. Note a simple numerical
%integration routine is use to compute angle vs time
dt = (1/250) * 2*pi*L/u;

%Define update frame rate /frames per second
FPS = (1/1) /dt;
%FPS = 24;

%Fontsize for graphs
fsize = 16;

%%

% Compute dynamics
[ T,v,theta,x,y, t, status, E ] = conker_calc( tmax, dt, g, L, m, u );

% Static diagnostic plots
figure('color', [1 1 1]);

%Status
subplot(2,3,1);
plot(t,status);
xlabel('t /s');
ylabel('status')
title({'0 circular motion under tension','1 projectile'} )
axis tight
ylim( [0,1])
set( gca, 'ytick', [0,1] );

%Velocity
subplot(2,3,2)
plot(t,v);
xlabel('t /s')
ylabel('v /ms^{-1}')
title('Velocity')
axis tight
ylim( [0, max(v)] )

%Tension
subplot(2,3,3)
plot(t,T);
xlabel('t /s')
ylabel('T /N')
title('String tension')
axis tight
ylim( [0, max(T)] )

%Energy
subplot(2,3,4)
plot(t,E);
xlabel('t /s')
ylabel('E /J')
title('Total energy')
axis tight
ylim( [0, max(E)] )

%Range
subplot(2,3,5)
r=sqrt(x.^2 + y.^2);
plot(t,r);
xlabel('t /s')
ylabel('Range /m')
title('Range')
axis tight
ylim( [0, max(r)] )

%Angle
subplot(2,3,6)
plot(t,theta);
xlabel('t /s')
ylabel('theta / radians')
title('Angle')
axis tight
ylim( [min(theta), max(theta)] )

%Save PNG image of the figure and close
print( gcf, 'conker_diagnostics.png', '-r300','-dpng' )
close(gcf);

%Interpolate data at frame rate
ti = linspace( 0, tmax, ceil(tmax*FPS) );
theta = interp1( t, theta, ti );
T = interp1( t, T, ti );
v = interp1( t, v, ti );
x = interp1( t, x, ti );
y = interp1( t, y, ti );
status = interp1( t, status, ti );
E = interp1( t, E, ti );
t = ti;

% Animation. Continue until a key is pressed
fig = figure( 'color', [1 1 1],'name','conker simulation');
N = length(T);
axes;
view(2);
xlim( 1.1*[-L,L] );
ylim( 1.1*[-L,L] );
set(gca, 'fontsize',fsize);
xlabel('x','fontsize',fsize)
ylabel('y','fontsize',fsize)
axis equal
axis manual;
grid on
hold on;
n = 1;
pend = plot( x(1),y(1),'marker','*' );
pstring = plot( [ 0, x(1)], [0, y(1)], 'b' );
ptraj = plot( x(1),y(1),'color','m' );
tit = title(' ');
while ishandle(fig) == 1
    
    %Update plot
    if status(n) == 0
        set( pend,'xdata',x(n),'ydata',y(n),'color','r');
    else
        set( pend,'xdata',x(n),'ydata',y(n),'color','g');
    end
    set( pstring,'xdata',[0,x(n)],'ydata',[0,y(n)]);
    set( ptraj,'xdata',x(1:n),'ydata',y(1:n));
    set(tit, 'string', {['t = ',num2str(t(n),2),...
        ' s. T = ',num2str(T(n),3),' N. ',...
        'v = ',num2str(v(n),2),' ms^{-1}. '],
        ['Simulation ',num2str(100*t(n)/tmax,2),'% complete.']} )
    drawnow;
    
    %Update frame counter
    if n==N
        %Reset animation
        n = 1;
    else
        %Update n
        n = n + 1;
    end
    
    %Check if a key has been pressed. If so, close figure
    try
        if isempty( get(fig,'Currentkey') )==0
            close(fig);
        end
    end
    
    %Pause before next frame
    pause(1/FPS);
end

%Save a final screen shot, then close
if ishandle(fig) == 1
    print( fig, 'conker_animation.png', '-r300','-dpng' )
    close(fig);
end

%%

% Function which calculates conker dynamics
function [ T,v,theta,x,y, t, status, E ] = conker_calc( tmax, dt, g, L, m, u, FPS )

%Turn off warnings
warning off;

%Determine linearly spaced time vector /s
N = ceil(tmax/dt) + 1 ;
t = linspace( 0, tmax, N);

%Initialise current values
theta = 0;
theta_dot = u/L;
T = tension( m, u, L, g, theta );
x = 0;
y = -L;
vx = u;
vy = 0;
v = sqrt( vx^2 + vy^2 );
r = sqrt( x^2 + y^2 );
status = 0;
tt=0;

%Initialise outputs vectors
theta_s = [theta,zeros(1,N-1)];
T_s = [T,zeros(1,N-1)];
v_s = [v,zeros(1,N-1)];
x_s = [x,zeros(1,N-1)];
y_s = [y,zeros(1,N-1)];
status_s = [status,zeros(1,N-1)];

%Use a discrete time numerical method to compute dynamics of conker
for n=2:N
    if status == 0
        %Constant radius motion under string tension
        [ theta, theta_dot ] = verlet_theta( theta, theta_dot, dt, L, g );
        x = L*sin(theta);
        y = -L*cos(theta);
        vx = L*theta_dot*cos(theta);
        vy = L*theta_dot*sin(theta);
        v = sqrt( vx^2 + vy^2 );
        T = tension( m, u, L, g, theta );
    else
        %Projectile motion, no string tension
        vy = vy -g*tt;
        v = sqrt( vx^2 + vy^2 );
        x = x + vx*tt;
        y = y + vy*tt -0.5*g*tt^2;
        r = sqrt( x^2 + y^2 );
        ang = atan2(y,x);
        if ang<0
            ang = ang+2*pi;
        end
        ang = pi/2 + ang;
        dtheta = ang - theta;
        theta = theta + dtheta;
        theta_dot = ( vx*cos(theta) + vy*sin(theta) ) / r;
        tt = tt + dt;
    end
    
    %Check new radius and reset motion status if needed. i.e. change
    %projectile to constant radius status. Reset projectile motion clock
    if r >= L
        %Constant radius = L motion
        r = L;
        x = L*sin(theta);
        y = -L*cos(theta);
        status = 0;
        tt = 0;
    end
    
    %Change status from constant radius to projectile if tension is less
    %than zero
    if T < 0
        %Change to projectile motion, start clock
        status = 1;
        T = 0;
    end
    
    %Store updated variables
    theta_s(n) = theta;
    T_s(n) = T;
    v_s(n) = v;
    x_s(n) = x;
    y_s(n) = y;
    status_s(n) = status;
end

%Define vector outputs
theta = unwrap(theta_s);
T = T_s;
v = v_s;
x = x_s;
y = y_s;
status = status_s;
E = 0.5*m*v.^2 + m*g*(y+L);

%%

% Calculate string tension
function T = tension( m, u, L, g, theta )
T = m * ( (u^2)/L -2*g + 3*g*cos(theta) );

%%

% Update theta in constant radius scenario based upon Verlet algorithm
function [ theta, theta_dot ] = verlet_theta( theta, theta_dot, dt, L, g )
a = g/L;
theta_double_dot = -a*sin(theta);
theta = theta + theta_dot*dt + 0.5*theta_double_dot*dt^2 ;
theta_double_dot_dt = -a*sin(theta);
theta_dot = theta_dot + 0.5 * ( theta_double_dot_dt + theta_double_dot )*dt;

%%

% Calculate conker speed using conservation of energy
function v = speed( u, L, g, theta )
v = sqrt( u^2 -2*g*L*( -cos(theta) + 1 ) );

%End of code