%dpendulum
% Script which computes the dynamics of a double pendulum given 
% defined initial conditions. Dynamical variables a1, a2 are the 
% angles of each pendulum bob's rod measured anticlockwise from the
% vertical.
%
% LAST UPDATED by Andy French April 2016

function dpendulum

%%% INPUT PARAMETERS %%%

%Bob masses /kg
m1=1;
m2=4;

%Rigid connecting rod lengths (negligible mass) /m
l1=3;
l2=1;

%Gravitational acceleration /ms^-2
g=9.81;

%Duration of solution set /s  (10 frames per second is not a bad idea)
T=10;

%Solver flag. If msolve=1, use MATLAB's built in ode45 ODE solver. If =0,
%use rungekutta.m using the time-step defined in 'timestep'
msolve=0;
timestep=0.01;

%

%%% INITIAL CONDITIONS %%%

%Initial bob rod angles in degrees. (Measured anticlockise from the
%vertical).
a1_init=160;
a2_init=160;

%Initial bob angular velocities /degrees per second
adot1_init=-180;
adot2_init=360;

%

%%% OUTPUT LIST %%%

%Plot bob rod angles vs time
plot_a=0;

%Plot bob X and Y values vs time
plot_xyt=0;

%Plot X,Y trajectory
plot_xy=1;

%Generate N PNG files for generation of an animated GIF
%This includes a representation of the pendulum using shaded spheres!
plot_png=1;
N=20;

%Plot Kinetic and Potential energies vs time
plot_ke_pe=1;

%Plot angular momenta vs time (Note since motion is defined to be planar, this will
%be a scalar rather than vector quantity - the actual direction is the Z
%Cartesian axis)
plot_j=0;

%Plot bob angular velocity and angular acceleration vs time
plot_adot_addot=0;

%Plot Cartesian X and Y velocities and X and Y trajectories vs time
plot_xydot_xyddot=0;

%Plot bob speeds vs time
plot_speed=0;

%Draw 'Poincare diagram' of bob angle vs angular acceleration, for each bob
plot_poincare=1;

%

%%% SOLVE! %%%

%Display opening message to command window
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Double Pendulum Simulation ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Compute bob angles a1 and a2 in degrees for times within [0,T] as well as
%bob Cartesian X,Y positions, angular velocities / degrees per second,
%angular acceleration /degrees per second per second
%Cartesian X and Y velocities, Cartesian X and Y accelerations, Angular
%momentum, Kinetic Energy and Gravitational Potential Energy.
[t,a1,a2,x1,y1,x2,y2,adot1,adot2,addot1,addot2,xdot1,ydot1,...
    xdot2,ydot2,xddot1,yddot1,xddot2,yddot2,...
    j1,j2,ke1,ke2,pe1,pe2]=solve_dpendulum(m1,m2,l1,l2,g,T,...
    msolve,timestep,a1_init,a2_init,adot1_init,adot2_init);

%

%%% SAVE & PLOT RESULTS %%%

%Save data to a .mat file
save 'double_pendulum';

%Plot graphs
plot_dpendulum(plot_a,plot_xyt,plot_xy,plot_png,plot_ke_pe,...
    plot_j,plot_adot_addot,plot_xydot_xyddot,plot_speed,plot_poincare,...
    t,a1,a2,x1,y1,x2,y2,adot1,adot2,addot1,addot2,xdot1,ydot1,...
    xdot2,ydot2,xddot1,yddot1,xddot2,yddot2,...
    j1,j2,ke1,ke2,pe1,pe2,...
    m1,m2,l1,l2,g,T,...
    msolve,timestep,a1_init,a2_init,adot1_init,adot2_init,N);

%%

%TOTAL_ENERGY
% Compute total energy E of Double Pendulum system
%
% LAST UPDATED by Andy French 11-Dec-2005
%
% a1      - Angle of bob 1 (radians) from vertical
% a2      - Angle of bob 1 (radians) from vertical
% adot1   - Angular velocity of bob 1
% adot2   - Angular velocity of bob 2
% m1      - Mass of bob 1 /kg
% m2      - Mass of bob 2 /kg
% l1      - Length of rigid string joining fixed pivot to bob 1 /m
% l2      - Length of rigid string joining bob 1 to bob 2 /m
% g       - Gravitational field strength /ms^-2
%
% E       - Total energy of system /joules
% A,B,C   - E = A + B*adot2*adot2 + C*adot2

function [E,A,B,C]=total_energy(a1,a2,adot1,adot2,m1,m2,l1,l2,g)

%Compute coefficients of quadratic expression in adot2
A=0.5*(m1+m2)*l1*l1*adot1.*adot1  -(m1+m2)*g*l1*cos(a1) - m2*g*l2*cos(a2);
B=0.5*m2*l2*l2;
C=m2*l1*l2*adot1.*cos(a1-a2);

%Compute total energy
E=A + B*adot2.*adot2 + C.*adot2;

%%

%THELEGEND
% Function which simply creates a legend, without the need to refer to a
% plot, or a set of plot handles.
%
% LAST UPDATED by Andy French. 10-Dec-2005
%
% Syntax: h=thelegend(strings,chars)
%
% strings  - Cell array of legend strings
% chars    - Cell array of corresponding plot symbols to elements of
%            strings
%
% h        - Legend handle
%
% Example: h=thelegend({'moo','oom'},{'r.-','b*'})

function h=thelegend(strings,chars)

%Find length of strings
L=length(strings);

%Plot characters at the origin
for n=1:L
ph(n)=plot(0,0,chars{n});
hold on
end

%Create legend
h=legend(ph,strings);

%Delete origin characters
delete(ph);

%%

%SOLVE_DPENDULUM
% Function which solves the dynamics of a double pendulum, i.e. computes
% the time history of the angles between the two rods of the pendulum and
% the vertical. Other dynamic parameters such as kinetic energy and angular
% momentum are also computed.
%
% LAST UPDATED by Andy French. 11-Dec-2005.
%
% Syntax: [t,a1,a2,x1,y1,x2,y2,adot1,adot2,addot1,addot2,xdot1,ydot1,...
%     xdot2,ydot2,xddot1,yddot1,xddot2,yddot2,...
%     j1,j2,ke1,ke2,pe1,pe2]=solve_dpendulum(m1,m2,l1,l2,g,T,...
%     msolve,timestep,a1_init,a2_init,adot1_init,adot2_init);
%
% m1            - Mass of pendulum bob 1 /kg
% m2            - Mass of pendulum bob 2 /kg
% l1            - Length of the (rigid) string joining the fixed pivot
%                 point to bob 1 /m
% l2            - Length of the (rigid) joining bob2 to bob 1 /m
% g             - Gravitational field strength /ms^-2
% T             - Duration of dynamics /s
% msolve        - Choose solver. ==1 MATLAB's ode45 solver
%                 ==0 4th order Runge-Kutta method
% timestep      - Timestep between solution evaluation
% a1_init       - Initial (clockwise) angle /degrees of bob 1 from vertical
% a2_init       - Initial (clockwise) angle /degrees of bob 2 from vertical
% adot1_init    - Initial angular velocity of bob 1 /degrees per second
% adot2_init    - Initial angular velocity of bob 2 /degrees per second
% 
% t             - Time /s
% a1            - Angle (clockwise) of bob 1 from vertical /degrees
% a2            - Angle (clockwise) of bob 2 from vertical /degrees
% x1            - Cartesian X coordinate of bob 1 /m
% y1            - Cartesian Y coordinate of bob 1 /m
% x2            - Cartesian X coordinate of bob 2 /m
% y2            - Cartesian Y coordinate of bob 2 /m
% adot1         - Angular velocity of bob 1 / deg s^-1
% adot2         - Angular velocity of bob 2 / deg s^-1
% addot1        - Angular acceleration of bob 1 /deg s^-2
% addot2        - Angular acceleration of bob 2 /deg s^-2
% xdot1         - Cartesian X velocity of bob 1 /ms^-1
% ydot1         - Cartesian Y velocity of bob 1 /ms^-1
% xdot2         - Cartesian X velocity of bob 2 /ms^-1
% ydot2         - Cartesian Y velocity of bob 2 /ms^-1
% xddot1        - Cartesian X acceleration of bob 1 /ms^-2
% yddot1        - Cartesian Y acceleration of bob 1 /ms^-2
% xddot2        - Cartesian X acceleration of bob 2 /ms^-2
% yddot2        - Cartesian Y acceleration of bob 2 /ms^-2
% j1            - Angular momentum (Z projection) of bob 1 /kg m^2 s^-1
% j2            - Angular momentum (Z projection) of bob 2 /kg m^2 s^-1
% ke1           - Kinetic energy of bob 1 /Joules
% ke2           - Kinetic energy of bob 2 /Joules
% pe1           - Gravitational potential energy of bob 1 /Joules
% pe2           - Gravitational potential energy of bob 2 /Joules

function [t,a1,a2,x1,y1,x2,y2,adot1,adot2,addot1,addot2,xdot1,ydot1,...
    xdot2,ydot2,xddot1,yddot1,xddot2,yddot2,...
    j1,j2,ke1,ke2,pe1,pe2]=solve_dpendulum(m1,m2,l1,l2,g,T,...
    msolve,timestep,a1_init,a2_init,adot1_init,adot2_init)

%Initliase outputs
t=[];
a1=[];
a2=[];
adot1=[];
adot2=[];

%Convert initial angles and angular velocities to radian units
a1_init=a1_init*pi/180;
a2_init=a2_init*pi/180;
adot1_init=adot1_init*pi/180;
adot2_init=adot2_init*pi/180;

%Compute total energy (which is conserved)
[E_tot,A,B,C]=total_energy(a1_init,a2_init,...
    adot1_init,adot2_init,m1,m2,l1,l2,g);

%Solve in TT second steps
TT=100;

%Determine number of TT intervals
num_TT=ceil(T/TT);

for n=1:num_TT

    %Display message to command window
    disp(['Solver interation ',num2str(n),' \',num2str(num_TT)])

    %Determine start time
    tstart=(n-1)*TT;

    %Determine end time
    if n~=num_TT
        tend=n*TT;
    else
        tend=T;
    end

    %Assign initial conditions
    if n==1
        a1INIT=a1_init;
        a2INIT=a2_init;
        adot1INIT=adot1_init;
        adot2INIT=adot2_init;
    else
        a1INIT=a1(end);
        a2INIT=a2(end);
        adot1INIT=adot1(end);
        adot2INIT=adot2(end);
    end

    %Determine angles and angular velocities
    if msolve==1
        %Use MATLAB ODE solver

        %Define initialisation vector z0
        z0(1)=a1INIT;
        z0(2)=a2INIT;
        z0(3)=adot1INIT;
        z0(4)=adot2INIT;

        %Call solver ode45 using inline function dpendulum_func defined below
        [tt,z]=ode113(@dpendulum_func,[tstart,tend],z0.',[],m1,m2,l1,l2,g,E_tot);

        %Interpolate to yield results equispaced by timestep
        tt_equispace=tstart:timestep:tend;  
        zz=zeros(length(tt_equispace),4);
        for k=1:4
            zz(:,k)=interp1(tt.',z(1:end,k),tt_equispace.');
        end
        tt=interp1(tt,tt,tt_equispace.');

        %Assign output vectors (angles in radians still)
        a1=[a1,zz(2:end,1).'];
        a2=[a2,zz(2:end,2).'];
        adot1=[adot1,zz(2:end,3).'];
        adot2=[adot2,zz(2:end,4).'];
        t=[t,tt(2:end).'];

    else
        %Use (supplied) Runge-Kutta solver to
        %determine angles /radians and angular velocities /radians per second
        [tt,aa1,aa2,aadot1,aadot2]=rungekutta(m1,m2,l1,l2,g,[tstart,tend],timestep,...
            a1INIT,a2INIT,adot1INIT,adot2INIT);
        t=[t,tt(2:end)];
        a1=[a1,aa1(2:end)];
        a2=[a2,aa2(2:end)];
        adot1=[adot1,aadot1(2:end)];
        adot2=[adot2,aadot2(2:end)];
    end
end

%Compute Cartesian X and Y coordinates /metres. Origin is set at the fixed point
%from which the pendulum hangs
x1=l1*sin(a1);
y1=-l1*cos(a1);
x2=x1 + l2*sin(a2);
y2=y1 - l2*cos(a2);

%Compute Cartesian velocities /metres per second
xdot1=l1*cos(a1).*adot1;
ydot1=l1*sin(a1).*adot1;
xdot2=xdot1 +l2*cos(a2).*adot2;
ydot2=ydot1 + l2*sin(a2).*adot2;

%Compute angular accelerations /radians per second per second
[addot1,addot2]=compute_addot(m1,m2,l1,l2,g,a1,a2,adot1,adot2);

%Compute Cartesian accelerations /metres per second per second
xddot1=l1*( cos(a1).*addot1 - sin(a1).*adot1.*adot1);
yddot1=l1*( sin(a1).*addot1 + cos(a1).*adot1.*adot1);
xddot2=xddot1 + l2*( cos(a2).*addot2 - sin(a2).*adot2.*adot2);
yddot2=yddot1 + l2*( sin(a1).*addot2 + cos(a2).*adot2.*adot2);

%Compute angular momenta /kg * m^2 * s^-1
%projected onto the Cartesian Z direction
j1=m1*dot([zeros(size(x1));zeros(size(x1));ones(size(x1))],...
    cross([x1;y1;zeros(size(x1))],[xdot1;ydot1;zeros(size(x1))],1),1);
j2=m2*dot([zeros(size(x1));zeros(size(x1));ones(size(x1))],...
    cross([x2;y2;zeros(size(x1))],[xdot2;ydot2;zeros(size(x1))],1),1);

%Compute Kinetic and Gravitational Potential energies /Joules
ke1=0.5*m1*( xdot1.*xdot1 + ydot1.*ydot1 );
ke2=0.5*m2*( xdot2.*xdot2 + ydot2.*ydot2 );
pe1=m1*g*y1;
pe2=m2*g*y2;

%Assign angular outputs in degrees, with angles wrapped onto [-180,180]
a1=wrap_angle(a1)*180/pi;
a2=wrap_angle(a2)*180/pi;
adot1=adot1*180/pi;
adot2=adot2*180/pi;
addot1=addot1*180/pi;
addot2=addot2*180/pi;

%%

%Wrap angle to [-pi,pi]
function theta_wrap=wrap_angle(theta)
x=cos(theta);
y=sin(theta);
theta_wrap=atan2(y,x);

%%

%Inline function for use with solver
function dz_by_dt=dpendulum_func(t,z,m1,m2,l1,l2,g,E)

%Assign elements of z
a1=z(1);
a2=z(2);
adot1=z(3);
adot2=z(4);

%Compute angular accelerations /radians per second per second
[addot1,addot2]=compute_addot(m1,m2,l1,l2,g,z(1),z(2),z(3),z(4));

% %Compute angular velocity adot2 using conservation of energy
% [E,A,B,C]=total_energy(a1,a2,adot1,adot2,m1,m2,l1,l2,g);
% adot2=(-C + sqrt(C*C -4*B*(A-E)))/(2*B);

%Initialise and assign outputs
dz_by_dt=zeros(4,1);
dz_by_dt(1)=adot1;
dz_by_dt(2)=adot2;
dz_by_dt(3)=addot1;
dz_by_dt(4)=addot2;

%%

%RUNGEKUTTA
% Basic numerical solver for sets of coupled first order differential
% equations. Note MATLAB's own solvers, such as ode45, are much more
% sophisticated. In the case of the latter, a Runge-Kutta algorithm is
% used, but an adaptive timestep (or 'x' variable interval) is used.
%
% In this implementation, the set of coupled ODE's are represented by a
% vector equation
%
% dy/dt = f(t,y)
%
% where y is a column vector whose rows are the variables required to be
% integrated. Rather than using an inline function (like ode45), the
% function f(t,y) will be 'hard wired' into this solver. Therefore it will
% need to be adapted to any other ODE set.
%
% LAST UPDATED by Andy French. 10-Dec-2005
%
% Syntax: [t,a1,a2,adot1,adot2]=rungekutta(m1,m2,l1,l2,g,tlim,timestep,...
%    a1_init,a2_init,adot1_init,adot2_init)
%
% m1          - Mass of bob 1 /kg
% m2          - Mass of bob 2 /kg
% l1          - Length of the (rigid) string joining the fixed pivot
%               point to bob 1 /m
% l2          - Length of the (rigid) joining bob2 to bob 1 /m
% g           - Gravitational field strength /ms^-2
% tlim        - Two element vector [tstart,tend] defining start and end 
%               times /s of timescale that dynamics are computed over 
% timestep    - Time interval between outputs /s
% a1_init     - Initial (clockwise) angle of bob 1 measured from the
%               vertical /radians
% a2_init     - Initial (clockwise) angle of bob 2 measured from the
%               vertical radians
% adot1_init  - Initial angular velocity of bob 1 /radians s^-1
% adot2_init  - Initial angular velocity of bob 2 /adians s^-1
%
% t           - Time /s
% a1          - (clockwise) angle of bob 1 measured from the
%               vertical /radians
% a2          - (clockwise) angle of bob 2 measured from the
%               vertical /radians
% adot1       - Angular velocity of bob 1 /radians s^-1
% adot2       - Angular velocity of bob 2 /radians s^-1

function [t,a1,a2,adot1,adot2]=rungekutta(m1,m2,l1,l2,g,tlim,timestep,...
    a1_init,a2_init,adot1_init,adot2_init)

%Define time /seconds
t=tlim(1) : timestep : tlim(2);

%Define initial conditions vector y. Angles must be in radians.
y0=[a1_init;...
    a2_init;...
    adot1_init;...
    adot2_init];

%Set first columns of output vector y to be the input
y=zeros(length(y0),length(t));
y(:,1)=y0(:);

%Evaluate Runge-Kutta parameters and iteratively solve for y
h=timestep;
for n=2:length(t)
    k1=h*f(t(n-1),y(:,n-1),m1,m2,l1,l2,g);
    k2=h*f(t(n-1)+0.5*h,y(:,n-1)+0.5*k1,m1,m2,l1,l2,g);
    k3=h*f(t(n-1)+0.5*h,y(:,n-1)+0.5*k2,m1,m2,l1,l2,g);
    k4=h*f(t(n-1)+h,y(:,n-1)+k3,m1,m2,l1,l2,g);
    y(:,n)=y(:,n-1) + k1/6 + k2/3 + k3/3 + k4/6 ;
end

%Assign outputs. Angles in radians
a1=y(1,:);
a2=y(2,:);
adot1=y(3,:);
adot2=y(4,:);

%%

%Define differential equation function
function dy_by_dt = f(t,y,m1,m2,l1,l2,g)

%Compute angular acceleations
[addot1,addot2]=compute_addot(m1,m2,l1,l2,g,y(1,:),y(2,:),y(3,:),y(4,:));

%Assign outputs
dy_by_dt(1,:)=y(3,:);
dy_by_dt(2,:)=y(4,:);
dy_by_dt(3,:)=addot1;
dy_by_dt(4,:)=addot2;

%%

%PLOT_DPENDULUM
% Function which geneates a selection of plots which illustrate the
% dynamics of the Double Pendulum
%
% LAST UPDATED by Andy French 10-Dec-2005

function plot_dpendulum(plot_a,plot_xyt,plot_xy,plot_png,plot_ke_pe,...
    plot_j,plot_adot_addot,plot_xydot_xyddot,plot_speed,plot_poincare,...
    t,a1,a2,x1,y1,x2,y2,adot1,adot2,addot1,addot2,xdot1,ydot1,...
    xdot2,ydot2,xddot1,yddot1,xddot2,yddot2,...
    j1,j2,ke1,ke2,pe1,pe2,...
    m1,m2,l1,l2,g,T,...
    msolve,timestep,a1_init,a2_init,adot1_init,adot2_init,num_frames)

%Plot and save frames of pendulum animation. Save TIFF files to a directory
%/Animation
if plot_png==1
    
    %If directory /animation already exists, delete it!
    if exist([pwd,'\dpendulum animation'])
        disp('Deleting old animation directory ....')
        rmdir([pwd,'\dpendulum animation'],'s')
    end
    
    %Create animation directory
    animdir=[pwd,'\dpendulum animation\1-99'];
    mkdir(animdir);
    
    %Compute frame times
    t_frame=linspace(0,T,num_frames);
    
    %Generate coordinates of a unit sphere
    [xs,ys,zs]=sphere(100);
    
    %Generate coordinates of a unit circle
    [xc,yc]=circle(200);
    
    %Scale sphere
    xs=xs*0.3;
    ys=ys*0.3;
    zs=zs*0.3;
    
    %Generate figure and axes
    figure('color',[1 1 1],'name','Double Pendulum',...
        'units','pixels','numbertitle','off')
    screendim=get(0,'screensize');
    set(gcf,'position',[screendim(3)/2-250,screendim(4)/2-250,500,500]);
    
    %Compute indices of t which are nearest to frame times and use this to
    %plot and save frames
    for n=1:num_frames
        disp( ['Double Pendulum frame ',num2str(n),' \ ',num2str(num_frames)] );
        pause(0.1)
        ti=approx_find(t,t_frame(n));
        set(gcf,'name',['Double Pendulum frame ',num2str(n),' \ ',num2str(num_frames)]);
        set(gca,'nextplot','replace')
        
        %Plot black sphere fixed point and allow other objects to be added
        %to axes
        fp=surf(xs,ys,zs);
        set(fp,'facecolor',[0 0 0],'edgecolor','none');
        hold on
        
        %Plot blue sphere for bob 1
        bob1=surf(xs + x1(ti),ys +y1(ti), zs);
        set(bob1,'facecolor',[0 0 1],'edgecolor','none');
        
        %Plot red sphere for bob 2
        bob2=surf(xs + x2(ti),ys +y2(ti), zs);
        set(bob2,'facecolor',[1 0 0],'edgecolor','none');
        
        %Plot lines joining fixed point to bobs 1 and 2
        plot([0,x1(ti),x2(ti)],[0,y1(ti),y2(ti)],'k')
        
        %Plot circles defining possible positions of bobs 1 and 2
        circle1=plot(xc*l1,yc*l1);
        set(circle1,'color',[0,0,1]);
        circle2=plot(xc*l2 + x1(ti),yc*l2 + y1(ti));
        set(circle2,'color',[1,0,0]);
        
        %Plot trajectory of bob 2
        plot(x2(1:ti),y2(1:ti),'g')
        
        %Set 2D view and axis limits
        view(2)
        axis equal
        xlim([-l1-2*l2,l1+2*l2])
        ylim([-l1-2*l2,l1+2*l2])
        zlim([-l1-2*l2,l1+2*l2])
        axis off
        camlight
        lighting phong
        
        %Set title, with time information
        title({'\bfDouble pendulum',...
            ['\rmm1=',num2str(m1),'kg  m2=',num2str(m2),...
            'kg  l1=',num2str(l1),' metres  l2=',num2str(l2),' metres'],...
            ['time =',num2str(floor(t(ti))),' s']});
        
        %If n exceeds an integer multiple of 100, create a new directory
        %Save indexed png frame to directory /animation
        if (n/100)==round(n/100)
            animdir=[pwd,'\dpendulum animation\',num2str(n),' to ',num2str(n+99)];
            mkdir(animdir)
        end
        print( gcf, [animdir,'\frame',leadingzero(n,3),'.png'],'-dpng','-r300' );
    end
    disp('New animation frames written to disk!')
end

%%

%Plot bob rod angles from the vertical /degrees vs time /seconds
if plot_a==1
    figure('color',[1 1 1],'name','Double Pendulum bob rod angles')
    subplot(2,1,1)
    plot(t,a1)
    xlabel('Time /s')
    ylabel('Angle /deg')
    title('Pendulum bob rod 1 angle clockwise from vertical')
    subplot(2,1,2)
    plot(t,a2)
    xlabel('Time /s')
    ylabel('Angle /deg')
    title('Pendulum bob rod 2 angle clockwise from vertical')
    print(gcf, 'dpendulum t vs angle.png','-r300','-dpng' );
end

%%

%Plot bob Cartesian X and Y coordinates /metres vs time /s
if plot_xyt==1
    figure('color',[1 1 1],'name','Double Pendulum bob X and Y positions')
    subplot(2,1,1)
    plot(t,x1,t,x2)
    xlabel('Time /s')
    ylabel('x /metres')
    legend('bob1','bob2')
    title('Pendulum bob X positions')
    subplot(2,1,2)
    plot(t,y1,t,y2)
    xlabel('Time /s')
    ylabel('y /metres')
    legend('bob1','bob2')
    title('Pendulum bob Y positions')   
    print(gcf, 'dpendulum xyt.png','-r300','-dpng' );
end

%%

%Plot Cartesian X,Y trajectories of pendulum bobs
if plot_xy==1
    figure('color',[1 1 1],'name','Double Pendulum bob X,Y trajectories')
    p1=plot(x1,y1,'go');
    set(p1,'markersize',1,'markerfacecolor',[0,1,0])
    hold on
    p2=plot(x2,y2,'ko');
    set(p2,'markersize',1,'markerfacecolor',[0,0,0])
    xlabel('x /metres')
    ylabel('y /metres')
    title('Pendulum bob X,Y trajectories')
    tl=thelegend({'bob1','bob2'},{'g-','k-'});
    axis equal
    print(gcf, 'dpendulum xy.png','-r300','-dpng' );
end

%%

%Plot kinetic and potential energies
if plot_ke_pe==1
    figure('color',[1 1 1],'name','Double Pendulum kinetic and potential energies')
    subplot(2,2,1);
    plot(t,ke1,t,pe1)
    xlabel('Time /s')
    ylabel('Energy /joules')
    title('Pendulum bob 1 kinetic, potential energy')
    legend('Kinetic','Potential')
    axis tight
    subplot(2,2,2);
    plot(t,ke2,t,pe2)
    xlabel('Time /s')
    ylabel('Energy /joules')
    title('Pendulum bob 2 kinetic, potential energy')
    legend('Kinetic','Potential')   
    axis tight
    subplot(2,2,3);
    plot(t,ke1+pe1,t,ke2+pe2)
    xlabel('Time /s')
    ylabel('Energy /joules')
    title('Total energies, per bob')
    legend('Bob 1','Bob 2') 
    axis tight
    subplot(2,2,4);
    [E,A,B,C]=total_energy(a1*pi/180,a2*pi/180,...
        adot1*pi/180,adot2*pi/180,m1,m2,l1,l2,g);
    plot(t,ke1+pe1+ke2+pe2,t,E)
    xlabel('Time /s')
    ylabel('Energy /joules')
    title('Total energy of system')
    legend('Solver outputs','total\_energy.m')
    axis tight
    print(gcf, 'dpendulum energy.png','-r300','-dpng' );
end

%%

%Plot angular momentum projected onto Z axis
if plot_j==1
    figure('color',[1 1 1],'name','Double Pendulum angular momenta')
    plot(t,j1,t,j2,t,j1+j2)
    xlabel('Time /s')
    ylabel('Angular momentum /kgm^2s^{-1}')
    title('Angular momentum')
    legend('Bob 1','Bob 2','Total')
    print(gcf, 'dpendulum angular momentum.png','-r300','-dpng' );
end

%%

%Plot angular velocity and angular acceleration
if plot_adot_addot==1
    figure('color',[1 1 1],'name','Double Pendulum angular velocities and angular accelerations')
    subplot(2,1,1);
    plot(t,adot1,t,adot2)
    xlabel('Time /s')
    ylabel('Angular velocities / deg s^{-1}')
    title('Angular velocities')
    legend('Bob 1','Bob 2')
    subplot(2,1,2);
    plot(t,addot1,t,addot2)
    xlabel('Time /s')
    ylabel('Angular acceleration /deg s^{-2}')
    title('Angular acceleration')
    legend('Bob 1','Bob 2')  
    print(gcf, 'dpendulum adot.png','-r300','-dpng' );
end

%%

%Plot Cartesian velocities /ms^-1 and accelerations /ms^-2
if plot_xydot_xyddot==1
    figure('color',[1 1 1],'name',...
    'Double Pendulum X and Y velocities and accelerations')
    subplot(2,2,1)
    plot(t,xdot1,t,ydot1)
    xlabel('Time /s')
    ylabel('Velocity /ms^{-1}')
    title('Pendulum bob 1 X and Y velocities')
    axis tight
    subplot(2,2,2)
    plot(t,xddot1,t,yddot1)
    xlabel('Time /s')
    ylabel('Acceleration /ms^{-2}')
    title('Pendulum bob 2 X and Y acceleration')
    axis tight
    subplot(2,2,3)
    plot(t,xdot2,t,ydot2)
    xlabel('Time /s')
    ylabel('Velocity /ms^{-1}')
    title('Pendulum bob 1 X and Y velocities')
    axis tight
    subplot(2,2,4)
    plot(t,xddot2,t,yddot2)
    xlabel('Time /s')
    ylabel('Acceleration /ms^{-2}')
    title('Pendulum bob 2 X and Y acceleration')
    axis tight
    print(gcf, 'dpendulum xydot xyddot.png','-r300','-dpng' );
end

%%

%Plot pendulum bob speeds
if plot_speed==1
    figure('color',[1 1 1],'name','Double Pendulum bob speeds')
    plot(t,sqrt(xdot1.*xdot1 + ydot1.*ydot1),...
        t,sqrt(xdot2.*xdot2 + ydot2.*ydot2))
    xlabel('Time /s')
    ylabel('Speed /ms^{-1}')
    title('Double Pendulum bob speeds')
    legend('Bob 1','Bob 2')
    print(gcf, 'dpendulum speed.png','-r300','-dpng' );
end

%%

%Plot poincare diagrams. Plot bob angles against bob angular velocities
if plot_poincare==1
    figure('color',[1 1 1],'name','Double Pendulum Poincare Diagram')
    p1=plot(a1,adot1,'go');
    set(p1,'markersize',1,'markerfacecolor',[0,1,0])
    hold on
    p2=plot(a2,adot2,'ko');
    set(p2,'markersize',1,'markerfacecolor',[0,0,0])
    xlabel('Angle /deg')
    ylabel('Angular velocity /deg s^{-1}')
    title('Poincare diagram')
    tl=thelegend({'bob1','bob2'},{'g-','k-'});
    axis equal
    print(gcf, 'dpendulum poincare.png','-r300','-dpng' );
end

%%

%LEADINGZERO
% Function to return an N character string consisting of the integer part 
% of argument X with leading zeros as necessary
%
% Simon Gardner August 2000.
%
% Syntax:  outstring=leadingzero(integer array of indices (X) ,N)

function outstring = leadingzero(X,N)

outstring = [strrep(blanks(N-1 - fix(log10(X))),...
    ' ','0'),num2str(X,3)];

%%

%FIG2PIC 
% Converts a figure into a bitmap and saves this to a specified
% directory.
%
% LAST UPDATED by Andy French. 10-Dec-2005.
%
% Syntax: FIG2PIC(FIG,BITMAP_TYPE,NAME,DIRECTORY)
%
% FIG is the handle of the figure
% BITMAP_TYPE is a string specifying the type of bitmap to be saved
% out of 'png', 'jpeg', 'tiff','pbm', 'pgm', and 'ppm'
% NAME is the filename stringm, sans extension
% DIRECTORY is the path of the directory the file is to be saved into
% If DIRECTOY==[] then the local directory is used.
%
% Example: fig=figure; surf(peaks); fig2pic(fig,'png','example',[])

function fig2pic(fig,bitmap_type,name,directory)

%Convert figure window to image
[frame,map]=frame2im(getframe(fig));	

%Save file of this first plot
filename=[directory,'\',name,'.',bitmap_type];

%Write file to specified directory above
imwrite(frame,filename,bitmap_type); 

%%

%COMPUTE_ADDOT
% Computute angular accelerations /radians per second per second
% for an idealised double pendulum dynamic system.
%
% LAST UPDATED by Andy French. 11-Dec-2005
%
% Syntax: [addot1,addot2]=compute_addot(m1,m2,l1,l2,g,a1,a2,adot1,adot2)
%
% m1      - Mass of bob 1 /kg
% m2      - Mass of bob 2 /kg
% l1      - Length of the (rigid) string joining the fixed pivot point to bob 1 /m
% l2      - Length of the (rigid) joining bob2 to bob 1 /m
% g       - Gravitational field strength /ms^-2
% a1      - Angle of bob1 measured anticlockwise from the vertical /radians
% a2      - Angle of bob1 measured anticlockwise from the vertical /radians
% adot1   - Angular velocity of bob 1 /radians s^-1
% adot2   - Angular velocity of bob 2 /radians s^-1
%
% addot1  - Angular acceleration of bob 1 /radians s^-1
% addot2  - Angular acceleration of bob 2 /radians s^-1

function [addot1,addot2]=compute_addot(m1,m2,l1,l2,g,a1,a2,adot1,adot2)

if m1==0
    %Single pendulum. Bob acceleations equate - rigid string length of
    %l1+l2
    addot2=-(g/(l1+l2))*sin(a2);
    addot1=addot2;
else
    %Double pendulum
    U=m2*g*sin(a2).*cos(a1-a2)-m2*l1*adot1.*sin(a1-a2).*cos(a1-a2)-...
        g*(m1+m2)*sin(a1)-m2*l2*adot2.*adot2.*sin(a1-a2);
    V=-m2*l1*cos(a1-a2).*cos(a1-a2) + l1*(m1+m2);
    addot1=U./V;
    W=m2*l1.*adot1.*adot2.*sin(a1-a2) - m2*g*sin(a2) - m2*l1*addot1.*cos(a1-a2);
    addot2=W/(m2*l2);
end

%%

%CIRCLE
% Function that returns N vertices X,Y of a unit circle
%
% LAST UPDATED by Andy French 07-Dec-2005
%
% Syntax: [xc,yc]=circle(N)
%
% Example: [xc,yc]=circle(100);
% figure('color',[1 1 1],'name','Unit circle',...
%        'units','pixels','numbertitle','off')
% screendim=get(0,'screensize');
% set(gcf,'position',[screendim(3)/2-250,screendim(4)/2-250,500,500]);
% patch(xc,yc,'g'); 
% xlim([-2,2]); ylim([-2,2]); title('Unit circle')

function [xc,yc]=circle(N)

%Define radian angles beween 0 and 2*pi
a=linspace(0,2*pi,N);

%Define X and Y coordinates
xc=cos(a);
yc=sin(a);

%%

%APPROX_FIND returns the index of a vector element which is the 
% nearest match to the second argument of the function.
%
% Andy French. LAST UPDATED 27/07/2004.
%
% Syntax: OUTPUT=APPROX_FIND(VEC,ELEMENT)
%
% VEC is a vector of any length
% ELEMENT is a scalar
% OUTPUT is the nearest matching index of an element of VEC
% to ELEMENT
%
% Example: approx_find([1,2,3,4],2.1) yields 2

function output=approx_find(vec,element)

%Compute difference array
diff=abs(ones(size(vec))*element-vec);

%Find minimum difference and the corresponding element
min_diff=min(diff);
min_diff_index=find(diff==min_diff);

%Define output
output=min_diff_index(1);

%End of code
