%projectile
%
% Projectile simulator. Ignores air resistance! User inputs initial
% elevation, velocity, elevation step angle size and velocity step size,
% and strength of gravity and initial height.
% A parabola is then drawn. Elevation and velocity are modified by pressing
% the arrow keys. Left/Right controls the velocity, up/down controls the
% elevation.
%
% LAST UPDATED by Andy French 23/01/2014

function projectile

%Initialise global data store d which all sub-functions can read and
%write to
global d

%Define the number of data points in trajectory
d.N = 1000;

%Set font size of axes annotation
d.fsize = 14;

%Set step size for variable parameters
d.delev_deg = 1;
d.dh0 = 0.1;
d.dv = 0.5;

%

%% Command line interface %%

% Display welcome message to command line
disp(' ');
disp(' Welcome to PROJECTILE ! ' )
disp(' By Andy French February 2014' )
disp(' ')
disp(' Action keys:')
disp(' q             Quit ')
disp(' Up arrow      Increase elevation')
disp(' Down arrow    Decrease elevation')
disp(' Left arrow    Decrease velocity')
disp(' Right arrow   Increase velocity')
disp(' a             Increase launch height')
disp(' z             Decrease launch height')
disp(' p             Print screen')
disp(' r             Reset')
disp(' ')

%Special test input!
default = input(' Use defaults? ','s' );
if strcmp( default, 'Y' )
    disp(' ')
    disp( ' Default option selected ...' );
    d.elev_deg = 45;
    d.v0 = 10;
    d.g = 9.81;
    d.h0 = 1;
else
    
    %Request initial elevation /degrees
    disp(' ')
    disp(' Request initial elevation /degrees')
    d.elev_deg = str2num( input( ' elev_deg = ','s' ));
    
    %Request velocity /ms^-1
    disp(' ')
    disp(' Request velocity /ms^-1')
    d.v0 = str2num( input( ' v = ','s' ));
    
    %Request strength of gravity
    disp(' ')
    disp(' Request strength of gravity')
    d.g = str2num( input( ' g = ','s' ));
    
    %Request initial height in metres
    disp(' ')
    disp(' Request initial height in metres')
    d.h0 = str2num( input( ' h0 = ','s' ));
    
end

%Store original parameters
d.elev_deg_orig  = d.elev_deg;
d.h0_orig = d.h0;
d.v0_orig = d.v0;

%

%% Generate original trajectory %%

%Create figure and axes
d.fig = figure('name','projectile',...
    'renderer','opengl','color',[1,1,1],'InvertHardCopy', 'off','KeyPressFcn',@keypressfunc);
d.ax = axes;

%Generate surface
generate_trajectory;

%Figure out axes limits, and store these. Subsequent modifications to
%trajectory parameters will not change the limits.
d.xlimits = get( d.ax, 'xlim');
d.ylimits = get( d.ax, 'ylim');

%%

%Generate projectile trajectory
function generate_trajectory

%Allow read-write usage of global variable d
global d

%Convert elevation into radians
d.elev = d.elev_deg*pi/180;

%Get the parameters of the projectile parabola y = x^2
d.a = -0.5 * ( d.g/(d.v0^2) )*( 1 + tan(d.elev)^2 );
d.b = tan(d.elev);
d.c = d.h0;

%Work out the maximum value of x i.e. where the projectile impacts
d.xmax = (-d.b - sqrt( d.b^2 -4*d.a*d.c ) )/(2*d.a);

%Generate x,y coordinates of trajectory
d.x = linspace(0,d.xmax,d.N);
d.y = d.c + d.b*d.x + d.a*d.x.^2;

%Work out the height at apogee /m
d.x_apogee = ( d.v0^2 / (2*d.g) )*sin( 2*d.elev );
d.hmax = d.c + d.b*d.x_apogee + d.a*d.x_apogee.^2;

%Generate x velocities
d.vx = d.v0*cos(d.elev)*ones(1,d.N);

%Work out projectile time since launch /s
d.t = d.x./d.vx;

%Compute y velocity and speed v
d.vy = d.v0*sin(d.elev) - d.g*d.t;
d.v = sqrt( d.vx.^2 + d.vy.^2 );

%Work out impact velocity
d.vimpact = d.v(d.N);

%Work out impact angle /degrees form horizontal
d.impact_angle = (180/pi)*atan2( -d.vy(d.N),d.vx(d.N) );

%Plot the trajectory
axes(d.ax)
cla;
plot( d.x,d.y, 'linewidth', 2 );
grid on
xlabel('x /m','fontsize',d.fsize )
ylabel('y /m','fontsize',d.fsize)

%Set the font size of axis annotation
set( d.ax, 'fontsize', d.fsize );

%Construct a string for the graph title
row1 = ['Projectile trajectory: h0 = ',num2str(d.h0),'m, v0 = ',num2str(d.v0),'ms^{-1}, elev = ',num2str(d.elev_deg),'deg'];
row2 = ['x_{max} = ',num2str(d.xmax),'m, h_{max} = ',num2str(d.hmax),'m, flight time = ',num2str(d.t(end)),'s']; 
row3 = ['impact velocity = ',num2str(d.vimpact),'ms^{-1}, impact angle = ',num2str(d.impact_angle),'deg' ];
title( {row1,row2,row3}, 'fontsize',d.fsize)

%Plot y velocity vs x
% figure;
% plot( d.x, d.vy );
% xlabel('x /m')
% ylabel('vy /metres per second')

%%

%Key press handler
function keypressfunc( fig,evnt )

%Allow read-write usage of global variable d
global d

%Get the string of the current key press
s = get( d.fig,'currentkey');

%Actions, depending on the key pressed...

if strcmp(s,'q') == 1
    %Quit
    close(d.fig)   
else
    if strcmp(s, 'p') == 1
        %Print screen
        print(fig,['Projectile ',strrep(datestr(now),':','-')],'-djpeg','-r300');
        
    elseif strcmp(s, 'd') == 1
        %Default parameters
        d.elev_deg = d.elev_deg_orig;
        d.v0 = d.v0_orig;
        d.h0 = d.h0_orig;
        
    elseif strcmp(s, 'uparrow') == 1
        %Increase elevation
        d.elev_deg = d.elev_deg + d.delev_deg;
    elseif strcmp(s, 'downarrow') == 1
        %Decrease elevation
        d.elev_deg = d.elev_deg - d.delev_deg;
    elseif strcmp(s, 'rightarrow') == 1
        %Increase velocity
        d.v0 = d.v0 + d.dv;
    elseif strcmp(s, 'leftarrow') == 1
        %Decrease velocity
        d.v0 = d.v0 - d.dv;
    elseif strcmp(s, 'a') == 1
        %Increase initial height
        d.h0 = d.h0 + d.dh0;
    elseif strcmp(s, 'z') == 1
        %Decrease initial height
        d.h0 = d.h0 - d.dh0;
    end
    generate_trajectory;
    
    %Fix graph limits
    xlim( d.xlimits );
    ylim( d.ylimits );   
end

%End of code