%projectile
% Ignoring air resistance, this code solves the trajectory equation to
% determine the angle of elevation of a ballistic trajectory sufficient to
% strike a given target location. A fixed launch velocity and constant
% downward acceleration is assumed.
% 
% LAST UPDATED by Andy French April 2016

function projectile

%% INPUTS %%

% Acceleration due to gravity /ms^-2
g = 9.81;

% Launch velocity /ms^-1
u = 100;

% Height of launch site (y0) /m. Launch x coordinate x0 = 0
y0 = 0;

% Target x and y coordinates /m
xT =  800;
yT = 100;

% Number of data points to determine trajectory
N = 400;

% Title string string format
strf = '%3.1f';

% Font size
fsize = 18;

%%

%Solve trajectory equation for the angle of elevation needed to hit the
%target. There will be two solutions to this as the equation y(x) is a
%quadratic.

%Find angle of elevtion. Note if b^2 -4*a*c < 0, then there will be no real
%solution.
a = -0.5 * g * (xT^2) / u^2;
b = xT;
c = y0 - yT + a;
theta1 = atan( (-b + sqrt( b^2 -4*a*c )) / (2*a) );
if abs( imag(theta1) ) > 1e-6
    theta1 = NaN;
end
theta2 = atan( (-b - sqrt( b^2 -4*a*c )) / (2*a) );
if abs( imag(theta2) ) > 1e-6
    theta2  = NaN;
end

%Define vector of x coordinates
x = linspace( 0, xT, N );

%Find times /s
t1 = x/(u*cos(theta1));
t2 = x/(u*cos(theta2));
T1 = t1(end);
T2 = t2(end);

%Compute (x,y) trajectory
y1 = y0 + x*tan(theta1) - 0.5 * (1/u^2) * g * (1 + tan(theta1)^2 ) * x.^2 ;
y2 = y0 + x*tan(theta2) - 0.5 * (1/u^2) * g * (1 + tan(theta2)^2 ) * x.^2 ;

%Find velocities /ms^-1
v1_x = u*cos(theta1);
v1_y = u*sin(theta1) -g*t1;
v1 = sqrt( v1_x.^2 + v1_y.^2 );
v2_x = u*cos(theta2);
v2_y = u*sin(theta2) -g*t2;
v2 = sqrt( v2_x.^2 + v2_y.^2 );

%Plot trajectories

%x,y graph
figure('color',[1 1 1],'name','projectile')
plot( x,y1,x,y2);
hold on;
plot(0,y0,'g*')
plot(xT,yT,'r*')
xlabel('x /m','fontsize',fsize)
ylabel('y /m','fontsize',fsize)
t = title( char({ ['PROJECTILE: u = ',num2str(u,strf),'ms^{-1}'];...
    ['\theta_1 = ',num2str(theta1*180/pi,strf),'^o, T_1 = ',num2str(T1,strf),'s. v_1 = ',num2str(v1(end),strf),'ms^{-1}'];...
    ['\theta_2 = ',num2str(theta2*180/pi,strf),'^o, T_2 = ',num2str(T2,strf),'s. v_2 = ',num2str(v2(end),strf),'ms^{-1}'] }));
set(t,'HorizontalAlignment','left','fontsize',fsize);
set(gcf,'units','normalized','position',[0 0 1 1]);
set(gca,'fontsize',fsize);
grid on
legend({'\theta_1','\theta_2'},'location','bestoutside')

%Print PNG image
print('-dpng','-r300','projectile xy')

%x,y and x,v graph
figure('color',[1 1 1],'name','projectile')
set(gcf,'units','normalized','position',[0 0 1 1]);

%x,y graph
subplot(2,1,1)
plot( x,y1,x,y2);
hold on;
plot(0,y0,'g*')
plot(xT,yT,'r*')
xlabel('x /m','fontsize',fsize)
ylabel('y /m','fontsize',fsize)
t = title( char({ ['PROJECTILE: u = ',num2str(u,strf),'ms^{-1}'];...
    ['\theta_1 = ',num2str(theta1*180/pi,strf),'^o, T_1 = ',num2str(T1,strf),'s. v_1 = ',num2str(v1(end),strf),'ms^{-1}'];...
    ['\theta_2 = ',num2str(theta2*180/pi,strf),'^o, T_2 = ',num2str(T2,strf),'s. v_2 = ',num2str(v2(end),strf),'ms^{-1}'] }));
set(t,'HorizontalAlignment','left','fontsize',fsize);
set(gca,'fontsize',fsize);
grid on
legend({'\theta_1','\theta_2'},'location','bestoutside')

%x,v graph
subplot(2,1,2)
plot( x,v1,x,v2);
xlabel('x /m','fontsize',fsize)
ylabel('v /ms^{-1}','fontsize',fsize)
set(gca,'fontsize',fsize);
grid on
legend({'\theta_1','\theta_2'},'location','bestoutside')

%Print PNG image
print('-dpng','-r300','projectile xy xv')

%End of code

