%mp3process
% Applies simple sound processing to a .mp3 file. Options are:
%  - Normalization
%  - Reduction in amplitude
%  - Reverse
%  - Change playback speed
%
% LAST UPDATED by Andy French April 2016

function mp3process

%Add directory mp3 to path (including all subdirectories)
addpath(genpath('mp3'));

%Display message to command window
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%')
disp('  mp3 sound processor ')
disp(' %%%%%%%%%%%%%%%%%%%%')
disp(' ')

%Request processing option
disp(' Select sound processing option: ')
disp(' ')
disp(' Type 1 for NORMALIZATION ')
disp(' Type 2 for AMPLITUDE REDUCTION ' )
disp(' Type 3 for REVERSE ' )
disp(' Type 4 for CHANGE PLAYBACK SPEED' )
disp(' ')
option = input( ' > ' );

%Set default amplitude reduction and playback speeds
reduction = 0;
speed = 1;

%Obtain exra information, depending which option has been chosen
if isempty(option)==1
    
    %No option chosen (i.e. return key pressed)
    option = 3;
    disp(' REVERSE is the default option, which has now been chosen for you.')
    disp(' ');
    
elseif option == 2
    %Request % amplitude reduction
    disp(' ')
    disp(' Select % reduction (between 0 and 100%)' )
    reduction = input(' > ');
    if isempty(reduction) || (reduction < 0 ) || ( reduction > 100 )
        disp(' Option chosen was not sensible. Default of 80% has been chosen for you.')
        reduction = 80;
    end
    disp(' ');
    
elseif option == 4
    %Request playback speed change
    disp(' ')
    disp(' Select playback speed ratio. Note a negative number will reverse the sound.' )
    speed = input(' > ');
    if isempty(speed)
        disp(' Option chosen was not sensible. Default of 0.7 has been chosen for you.')
        speed = 0.7;
    end
    disp(' ');
end

%Determine file appendage depending on option chosen
if option == 1
    a = '_norm.mp3';
elseif option == 2
    a = ['_amp',num2str(100-reduction),'.mp3'];
elseif option == 3
    a = '_rev.mp3';
else
    a = ['_speed',num2str(speed),'.mp3'];
end

%Use standard Windows file navigation box to find .mp3 files. More than one
%can be selected.
disp(' Choose .mp3 file(s)...' )
[filename, pathname] = uigetfile( '*.mp3',...
    'Choose .mp3 files....', ...
    'MultiSelect', 'on');
if iscell(filename)==0
    if filename==0
        return
    else
        filename = {filename};
    end
end

%Start clock
t0 = tic;
t = 0;

%Step through filenames and process.
for n=1:length(filename)
    %Load .mp3 file (s is a vector of sound samples, sampled at
    %frequency Fs Hz, with maximum (signed) amplitude 2^(Nbits-1)
    file = [ pathname, filename{n} ];
    [s,Fs,Nbits] = mp3read( file );
    
    %Process and write a .mp3 file with _p file appendage
    s = process( s, option, reduction, speed );
    mp3write( s, Fs, strrep( file,'.mp3',a ) );
    
    %Determine current elapsed time
    tt = toc(t0);
    
    %Display completion message
    disp(' ')
    disp([' File ',num2str(n),' of ',num2str(length(filename)),' : ',filename{n}]);
    disp([' Processed in ', num2str(tt-t),' seconds.'])
    disp([' Total elapsed time is ',num2str(tt),' seconds.']);
    
    %Update current time to enable individual file conversion time
    %calculation
    t=tt;
end

%Display completion message
disp(' ')
disp(' .mp3 processing complete.')
disp(' ')

%Remove directory mp3 from path, including any subdirectories
rmpath(genpath('mp3'));

%%

%Sound processing effects
function s = process( s, option, reduction, speed )
if option ==1
    % NORMALIZE
    s = 0.999*s/max(max(abs(s)));
elseif option ==2
    % AMPLITUDE REDUCTION
    s = ( 1 - reduction/100 )*s;
elseif option == 3
    % REVERSE
    s = flipud(s);
elseif option == 4
    % CHANGE PLAYBACK SPEED
    if speed < 0
        s = flipud(s);
        speed = -speed;
    end
    dim = size(s);
    %Interpolate left and right stereo channels separately, then recombine.
    left = interp1( (1:dim(1)).', s(:,1),...
        linspace(1,dim(1), round( dim(1)/speed ) ).' );
    right = interp1( (1:dim(1)).', s(:,2),...
        linspace(1,dim(1), round( dim(1)/speed ) ).' );
    s = [left,right];
end

%End of code