%spectrogram
% Function which loads a .mp3 file and computes a spectrogram. This is a
% coloured plot which shows how the balance of frequencies changes with
% time.
% 
% LAST UPDATED by Andy French April 2016

function spectrogram

%Set fontsize for graph
fsize = 18;

%Define time window /s to determine 'frequency histogram' (i.e. a Short
%Time Fourier Transform or STFT)
dt = 0.1;

%Set maximum frequency /Hz to display
fmax = 5000;

%

%Add directory mp3 to path (including all subdirectories)
addpath(genpath('mp3'));

%Load .mp3 file
[filename, pathname] = uigetfile( '*.mp3',...
    'Choose .mp3 file to plot', ...
    'MultiSelect', 'off');
file = [ pathname, filename ];
[s,Fs,Nbits] = mp3read( file );

%Remove directory mp3 to path (including all subdirectories)
rmpath(genpath('mp3'));

%

%Compute Spectrogram

%Combine left and right channels and convert samples to a row vector
s = sum(s,2).';
N = length(s);

%Duration of recording /s
T = (numel(s)-1)/Fs;

%Determine number of STFTs
M = ceil( T/dt );

%Work out how many samples are required to add to N in order to divide by M exactly
pad = M - rem( N, M );

%Work out length of each STFT
L = (N+pad)/M;

%Determine time /s and frequency /Hz vectors associated with the spectrogram
t = linspace(0,T,M);
f = linspace(0,Fs/2,round(L/2));
[t,f] = meshgrid(t,f);

%Pad signal with zeros if required
if pad>0
    s = [ s, zeros(1, pad ) ];
end

%Transform signal vector into a matrix
s = reshape(s,[L,M] );

%Compute weighted STFT over rows for each column. Output result in
%decibels.
w = dolphcheb(L,40);
w = repmat(w.',[1,M]);
s = dB( fft( w.*s ,[],1 ) );

%Ignore -ve frequencies. Since signal only contains amplitude information
%it will be symmetric. Assume no frequencies are dominant which are over
%half the sample rate!
s = s(1:round(L/2),:);

%Plot spectrogram and save as a .png file
figure('color',[1 1 1],'name', 'spectrogram');
interp_colormap(1000);
surf(t,f,s);
view(2);
axis tight
ylim([0,fmax])
box on
shading interp;
colorbar('fontsize',fsize);
xlabel('Time /s','fontsize',fsize);
ylabel('Frequency /Hz','fontsize',fsize);
title(['Spectrogram of ',filename],'fontsize',fsize);
set(gca,'fontsize',fsize)
print( gcf, strrep( filename,'.mp3',' spectrogram .png' ),'-dpng','-r300' );

%%

%dolphcheb
% Function that computes Dolph-Chebyshev windowing function. The main role
% of the window (as with all windows) is to damp out the effects of the
% Gibbs phenomenon that results from truncation of an infinite series (e.g.
% a Discrete Fourier Transform). Note this window is complex
% and can therefore only really be visualised when the amplitude of the
% window DFT is plotted (in dB).
%
% LAST UPDATED by Andrew French. 27/08/2004.
%
% Syntax: w=dolphcheb(N,sl_level)
%
% N      - Length of window sl_level. This ifference between peak and
%          sidelobes /dB in the DFT of the window.
% w      - Window 'weights'. These are real, positive numbers.

function w=dolphcheb(N,sl_level)
gamma=sl_level/20;
if N>1
    M=N-1;
    alpha=cosh(acosh(10^gamma)/M);
    for n=1:M
        A(n)=((-1)^(n-1))*cos(M*acos(alpha*cos(pi*(n-1)/M)));
    end
    B=fft(A);
    B(1)=0.5*B(1);
    B(N)=B(1);
    w=B/max(B);
else
    w=1;
end

%%

%dB
% Function which converts a quantity to decibels i.e. a logarithmic scale.
% The maximum value is taken to be 0 dB.
function y = dB(x)
y = NaN(size(x));
x = abs(x);
maxx = max(max(x));
if maxx > 0
    y(x>0) = 10*log10( x(x>0) ) - 10*log10( maxx );
end

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code