%normalcalc_poisson_from_P
% If probability P( x < x* ) = k is known, and x* is also known, such that
% x ~ Po( lambda ), this program finds lambda.
% 
% A normal distribution approximation is used, incorporating a continuity
% correction.
%
% LAST UPDATED by Andy French May 2016

function normalcalc_poisson_from_P

%Define x value
x1 = 120;

%Define probability such that x < x1
P = 0.8;

%Fontsize for graph
fsize = 18;

%Number of data points for graph
N = 1000;

%

%Apply continuity correction to limit
x1 = x1 + 0.5;

%Solve quadratic for lambda
a = 1;
b = -( 2*x1 + ( sqrt(2)*erfinv( 2*P -1 ) )^2 );
c = x1^2;
L1 = ( -b + sqrt( b^2 -4*a*c ) )/(2*a);
L2 = ( -b - sqrt( b^2 -4*a*c ) )/(2*a);

%Determine correct value for lambda
if P>0.5
    %x1 must be greater than lambda
    lambda = min( [L1,L2] );
else
    %x1 must be less than lambda
    lambda = max( [L1,L2] );
end

%Determine mean and standard deviation of normal distribution used to
%approximate the Poisson distribution
mu = lambda;
sdev = sqrt(lambda);

%Plot graph
normalplotter( -inf, x1, mu, sdev, N, fsize );

%%

%Normal distribution plotting function
function normalplotter( x0, x1, mu, sdev, N, fsize )

%Calculate probability P( x0 < x < x1 ) using normal distribution
z0 = ( x0 - mu )/sdev;
z1 = ( x1 - mu )/sdev;
P = phi(z1) - phi(z0);

%Determine title string
title_str = {
    ['Normal distribution. \mu = ',num2str(mu),', \sigma = ',num2str(sdev) ],...
    ['P( ',num2str(x0),' < x < ',num2str(x1),' ) = ',num2str(P)]
    };

%Create normal distribution probability desity function
x = linspace( mu - 4*sdev, mu + 4*sdev, N );
y = ( 1/sqrt(2*pi*sdev^2) )*exp( -( (x-mu).^2 )/(2*sdev^2) );

%Determine x,y coordinates of green patch area to illustrate area under the
%normal distribution which corresponds to probability P.
if x0 < mu - 4*sdev
    x0 = mu - 4*sdev;
end
if x1 > mu + 4*sdev
    x1 = mu + 4*sdev;
end
xx = linspace( x0, x1, N );
yy = ( 1/sqrt(2*pi*sdev^2) )*exp( -( (xx-mu).^2 )/(2*sdev^2) );

%Plot graph and annotate this with the inputs
figure(gcf);
clf;
set( gcf, 'name','normal distribution calculator','color',[1 1 1],'numbertitle','off' );
p = patch( [xx(1),xx,xx(end)],[0,yy,0],'g' );
set(p,'facealpha',0.5);
hold on;
plot(x,y);
grid on;
xlabel('x','fontsize',fsize)
ylabel('pdf','fontsize',fsize)
title( title_str,'fontsize',fsize);
set(gca,'fontsize',fsize);
ylim( [0,1.2*max(y)] )
grid on;
box on;

%Save graph as a .png file
filename = ['Normal mu=',num2str(mu),' sigma=',num2str(sdev),...
    ' x0=',num2str(x0),' x1=',num2str(x1),' P=',num2str(P)];
print(gcf,[filename,'.png'],'-dpng','-r300')

%%

%Function which evaluates the integral under the standardized normal
%distribution ( i.e. mean 0, standard deviation unity) from -inf to x
function y = phi(x)
y = 0.5*( erf( x/sqrt(2) ) + 1 );

%End of code