%skewed_normal
% The Skew Normal distribution is a generalization of the Gaussian (Normal)
% distribution. This function plots the probability density function,
% cumulative distribution function, and samples from the distribution.
%
% http://en.wikipedia.org/wiki/Skew_normal_distribution
%
% LAST UPDATED by Andy French May 2016

function skewed_normal

%Define inputs
x_mean = 2;
x_std = 4;
x_skew = 0.5;

%Define data range for probability desity function (pdf) plot
x = linspace( -10, 20, 300 );

%Define number of samples from a skew-normal distribution. The pdf will be
%plotted as an overlay upon an area normalized histogram determined from
%random samples from the skew-normal distribution.
N = 5000;

%Fontsize
fsize = 18;

%

%Create skew-normal distribution
[ x_s, pdf, cdf ] = skewednormal( x_mean, x_std, x_skew, N, x );

%PDF & normalized histogram
figure('color',[1 1 1],'name','Skew-Normal Distribution');
edges = linspace( min(x), max(x), 50 ) ;
spacing = edges(2)-edges(1);
h = histc(x_s,edges);
h = h/( spacing*sum(h) );
b = bar( edges, h, 'histc' );
set(b,'facecolor','g','facealpha',0.5)
hold on;
plot( x, pdf );
xlabel('x','fontsize',fsize);
ylabel('PDF','fontsize',fsize)
title({'Skew-Normal PDF',['Mean=',num2str(x_mean),', STD=',num2str(x_std),...
    ', SKEW=',num2str(x_skew)]},'fontsize',fsize)
grid on
set(gca,'fontsize',fsize)
print(gcf,'skew-normal PDF','-dpng','-r300')

%CDF
clf;
plot( x, cdf );
xlabel('x','fontsize',fsize);
ylabel('CDF','fontsize',fsize)
title('Skew-Normal CDF','fontsize',fsize)
grid on
set(gca,'fontsize',fsize)
print(gcf,'skew-normal CDF','-dpng','-r300')
close(gcf);

%%

%skewednormal
% Generates the skew-normal distribution.
%
% Syntax: [ x_s, pdf, cdf ] = skewed_normal( x_mean, x_std, x_skew, N, x )
%
% x_mean   Mean value of distribution E[x]
% x_std    Square root of the variance of the distribution V[x]
% x_skew   E[ ( (x - x_mean)/x_std )^3 ]
% N        Number of data samples
% x        Values of x at which the pdf and cdf are evaluated
%
% x_s      Vector of samples from the distribution
% pdf      Probability desnity function of the distribution, evaluated at x
% cdf      Cumulative distribution function, evaluated at x
%
% NOTE: NaN values are returned for is abs( x_skew) > 1

function [ x_s, pdf, cdf ] = skewednormal( x_mean, x_std, x_skew, N, x )

%Check skew is in range  -1 < x_skew < 1
x_s = NaN(1,N);
pdf = NaN(1,N);
cdf = NaN(1,N);
if abs( x_skew) < 1
    
    %Numerically invert the skewer function using piecewise Hermite polynomials
    d = linspace(-1,1,500);
    g = skewer(d);
    delta = interp1( g, d, x_skew, 'pchip' );
    
    %Determine other parameters of the Skew Normal PDF from the inputs
    w = x_std/sqrt( 1- (2/pi)*delta^2 );
    a = sqrt( (delta^2)/(1-delta^2) );
    ee = x_mean - w*delta*sqrt(2/pi);
    
    %Determine pdf
    z = (x-ee)/w;
    pdf = (1/w)*(1/sqrt(2*pi))*exp( -0.5*z.^2 ).*( erf( a*z/sqrt(2) ) + 1 );
    
    %Determine cdf
    zz = linspace( -10, 10, 500 );
    pp = (1/w)*(1/sqrt(2*pi))*exp( -0.5*zz.^2 ).*( erf( a*zz/sqrt(2) ) + 1 );
    cdf = w*integrator( zz,pp,-10, z );
    ccdf = w*integrator( zz,pp,-10, zz );
    
    %Sample from the distribution ...
    
    %Determine N values of the CDF within the range [0,1]
    cdf_values = rand(1,N);
    
    %Interpolate cdf to find the corresponding x values
    [ccdf,i] = unique(ccdf);
    zz = zz(i);
    z_s = interp1( ccdf, zz, cdf_values, 'pchip' );
    x_s = w*z_s + ee;  
end

%%

%integrator
% Integration function using cubic splines.
%
% Syntax: I = integrator( xx, yy, x0, x )
%
% xx, yy     Samples from function y(x) which is being integrated. These
%            must be the same dimensions and have at least three elements each.
% x0         Starting point (i.e. lower limit) of the integral
% x          Upper limit of the integral. This can be an array of any
%            dimensions.
%
% I          Output of integral corresponindg to array x
%
% The function is based on the code written on pages 292 and 294
% of "Mastering Matlab 6" by Duane Hanselman & Bruce Littlefield.

function I = integrator( xx, yy, x0, x )

%Check inputs are the right size
if ( numel(xx) == numel(yy) ) && ( numel(xx) >= 3 )
    
    %Sort xx in ascending order
    [xx,i] = sort(xx);
    yy = yy(i);
    
    %Compute piecewise polynomial based upon cubic splines.
    pp=spline(xx,yy);
    
    %Take apart piecewise polynomial
    [br,co,npy,nco] = unmkpp(pp);
    
    %Scale factors for integration
    sf=nco:-1:1;
    
    %Integral coefficients
    ico=[co./sf(ones(npy,1),:) zeros(npy,1)];
    
    %Integral spline has higher order
    nco=nco+1;
    
    %Set integration constant of all splines to be zero
    ico(1,nco)=0;
    
    %Find constant terms in polynomials
    for k=2:npy
        ico(k,nco)=polyval(ico(k-1,:),br(k)-br(k-1));
    end
    
    %Build pp form for integral
    ppi=mkpp(br,ico);
    
    %Evaluate definite integral between limits of x0 to x
    I = ppval(ppi,x) - ppval(ppi,x0);
    
else
    I = NaN(size(x));
end

%%

%Returns the skew (y) in terms of skew-normal distribution shape parameter d
function y = skewer(d)
y = d.^3;
y = y.*( -(2/pi)*d.^2 + 1 ).^(-3/2);
y = 0.5*(4-pi)*y*( (2/pi)^(3/2) );

%End of code